# -*- coding: utf-8 -*-
import itertools
import requests
import datetime

from bs4 import BeautifulSoup
from flask import url_for
from flask_mail import Message
import query_phenomizer

from scout.constants import (CASE_STATUSES, PHENOTYPE_GROUPS, COHORT_TAGS, SEX_MAP, PHENOTYPE_MAP, VERBS_MAP)
from scout.constants.variant_tags import MANUAL_RANK_OPTIONS, DISMISS_VARIANT_OPTIONS, GENETIC_MODELS
from scout.server.utils import institute_and_case
from scout.parse.clinvar import clinvar_submission_header, clinvar_submission_lines
from scout.server.blueprints.variants.controllers import variant as variant_decorator
from scout.server.blueprints.variants.controllers import sv_variant

STATUS_MAP = {'solved': 'bg-success', 'archived': 'bg-warning'}


def cases(store, case_query, limit=100):
    """Preprocess case objects.

    Add the necessary information to display the 'cases' view

    Args:
        store(adapter.MongoAdapter)
        case_query(pymongo.Cursor)
        limit(int): Maximum number of cases to display

    Returns:
        data(dict): includes the cases, how many there are and the limit.
    """

    case_groups = {status: [] for status in CASE_STATUSES}
    for case_obj in case_query.limit(limit):
        analysis_types = set(ind['analysis_type'] for ind in case_obj['individuals'])

        case_obj['analysis_types'] = list(analysis_types)
        case_obj['assignees'] = [store.user(user_email) for user_email in
                                 case_obj.get('assignees', [])]
        case_groups[case_obj['status']].append(case_obj)
        case_obj['is_rerun'] = len(case_obj.get('analyses', [])) > 0
        case_obj['clinvar_variants'] = store.case_to_clinVars(case_obj['_id'])
    data = {
        'cases': [(status, case_groups[status]) for status in CASE_STATUSES],
        'found_cases': case_query.count(),
        'limit': limit,
    }
    return data


def case(store, institute_obj, case_obj):
    """Preprocess a single case.

    Prepare the case to be displayed in the case view.

    Args:
        store(adapter.MongoAdapter)
        institute_obj(models.Institute)
        case_obj(models.Case)

    Returns:
        data(dict): includes the cases, how many there are and the limit.

    """
    # Convert individual information to more readable format
    case_obj['individual_ids'] = []
    for individual in case_obj['individuals']:
        try:
            sex = int(individual.get('sex', 0))
        except ValueError as err:
            sex = 0
        individual['sex_human'] = SEX_MAP[sex]
        individual['phenotype_human'] = PHENOTYPE_MAP.get(individual['phenotype'])
        case_obj['individual_ids'].append(individual['individual_id'])

    case_obj['assignees'] = [store.user(user_email) for user_email in
                             case_obj.get('assignees', [])]

    # Fetch the variant objects for suspects and causatives
    suspects = [store.variant(variant_id) or variant_id for variant_id in
                case_obj.get('suspects', [])]
    causatives = [store.variant(variant_id) or variant_id for variant_id in
                  case_obj.get('causatives', [])]

    # Set of all unique genes in the default gene panels
    distinct_genes = set()
    case_obj['panel_names'] = []
    for panel_info in case_obj.get('panels', []):
        if not panel_info.get('is_default'):
            continue
        panel_obj = store.gene_panel(panel_info['panel_name'], version=panel_info.get('version'))
        distinct_genes.update([gene['hgnc_id'] for gene in panel_obj.get('genes', [])])
        full_name = "{} ({})".format(panel_obj['display_name'], panel_obj['version'])
        case_obj['panel_names'].append(full_name)
    case_obj['default_genes'] = list(distinct_genes)

    for hpo_term in itertools.chain(case_obj.get('phenotype_groups', []),
                                    case_obj.get('phenotype_terms', [])):
        hpo_term['hpo_link'] = ("http://compbio.charite.de/hpoweb/showterm?id={}"
                                .format(hpo_term['phenotype_id']))

    # other collaborators than the owner of the case
    o_collaborators = [store.institute(collab_id) for collab_id in case_obj['collaborators'] if
                       collab_id != case_obj['owner']]
    case_obj['o_collaborators'] = [(collab_obj['_id'], collab_obj['display_name']) for
                                   collab_obj in o_collaborators]

    irrelevant_ids = ('cust000', institute_obj['_id'])
    collab_ids = [(collab['_id'], collab['display_name']) for collab in store.institutes() if
                  (collab['_id'] not in irrelevant_ids) and
                  (collab['_id'] not in case_obj['collaborators'])]

    events = list(store.events(institute_obj, case=case_obj))
    for event in events:
        event['verb'] = VERBS_MAP[event['verb']]

    case_obj['clinvar_variants'] = store.case_to_clinVars(case_obj['display_name'])

    # Phenotype groups can be specific for an institute, there are some default groups
    pheno_groups = institute_obj.get('phenotype_groups') or PHENOTYPE_GROUPS

    data = {
        'status_class': STATUS_MAP.get(case_obj['status']),
        'other_causatives': store.check_causatives(case_obj=case_obj),
        'comments': store.events(institute_obj, case=case_obj, comments=True),
        'hpo_groups': pheno_groups,
        'events': events,
        'suspects': suspects,
        'causatives': causatives,
        'collaborators': collab_ids,
        'cohort_tags': COHORT_TAGS,
    }
    return data


def case_report_content(store, institute_obj, case_obj):
    """Gather contents to be visualized in a case report

    Args:
        store(adapter.MongoAdapter)
        institute_obj(models.Institute)
        case_obj(models.Case)

    Returns:
        data(dict)

    """
    variant_types = {
        'causatives_detailed': 'causatives',
        'suspects_detailed': 'suspects',
        'classified_detailed': 'acmg_classification',
        'tagged_detailed': 'manual_rank',
        'dismissed_detailed': 'dismiss_variant',
        'commented_detailed': 'is_commented',
    }
    data = case_obj

    # Add the case comments
    data['comments'] = store.events(institute_obj, case=case_obj, comments=True)

    data['manual_rank_options'] = MANUAL_RANK_OPTIONS
    data['dismissed_options'] = DISMISS_VARIANT_OPTIONS
    data['genetic_models'] = dict(GENETIC_MODELS)
    data['report_created_at'] = datetime.datetime.now().strftime("%Y-%m-%d %H:%M")

    evaluated_variants = {}
    for vt in variant_types:
        evaluated_variants[vt] = []
    # We collect all causatives and suspected variants
    # These are handeled in separate since they are on case level
    for var_type in ['causatives', 'suspects']:
        #These include references to variants
        vt = '_'.join([var_type, 'detailed'])
        for var_id in case_obj.get(var_type,[]):
            variant_obj = store.variant(var_id)
            if not variant_obj:
                continue
            # If the variant exists we add it to the evaluated variants
            evaluated_variants[vt].append(variant_obj)

    ## get variants for this case that are either classified, commented, tagged or dismissed.
    for var_obj in store.evaluated_variants(case_id=case_obj['_id']):
        # Check which category it belongs to
        for vt in variant_types:
            keyword = variant_types[vt]
            # When found we add it to the categpry
            # Eac variant can belong to multiple categories
            if keyword in var_obj:
                evaluated_variants[vt].append(var_obj)

    for var_type in evaluated_variants:
        decorated_variants = []
        for var_obj in evaluated_variants[var_type]:
        # We decorate the variant with some extra information
            if var_obj['category'] == 'snv':
                decorated_info = variant_decorator(
                        store=store,
                        institute_obj=institute_obj,
                        case_obj=case_obj,
                        variant_id=None,
                        variant_obj=var_obj,
                        add_case=False,
                        add_other=False,
                        get_overlapping=False
                    )
            else:
                decorated_info = sv_variant(
                    store=store,
                    institute_id=institute_obj['_id'],
                    case_name=case_obj['display_name'],
                    variant_obj=var_obj,
                    add_case=False,
                    get_overlapping=False
                    )
            decorated_variants.append(decorated_info['variant'])
        # Add the decorated variants to the case
        data[var_type] = decorated_variants

    return data


def coverage_report_contents(store, institute_obj, case_obj, base_url):
    """Posts a request to chanjo-report and capture the body of the returned response to include it in case report

    Args:
        store(adapter.MongoAdapter)
        institute_obj(models.Institute)
        case_obj(models.Case)
        base_url(str): base url of server

    Returns:
        coverage_data(str): string rendering of the content between <body </body> tags of a coverage report
    """

    request_data = {}
    # extract sample ids from case_obj and add them to the post request object:
    request_data['sample_id'] = [ ind['individual_id'] for ind in case_obj['individuals'] ]

    # extract default panel names and default genes from case_obj and add them to the post request object
    distinct_genes = set()
    panel_names = []
    for panel_info in case_obj.get('panels', []):
        if not panel_info.get('is_default'):
            continue
        panel_obj = store.gene_panel(panel_info['panel_name'], version=panel_info.get('version'))
        full_name = "{} ({})".format(panel_obj['display_name'], panel_obj['version'])
        panel_names.append(full_name)
    panel_names = ' ,'.join(panel_names)
    request_data['panel_name'] = panel_names

    # add institute-specific cutoff level to the post request object
    request_data['level'] = institute_obj.get('coverage_cutoff', 15)

    #send get request to chanjo report
    resp = requests.get(base_url+'reports/report', params=request_data)

    #read response content
    soup = BeautifulSoup(resp.text)

    # remove links in the printed version of coverage report
    for tag in soup.find_all('a'):
        tag.replaceWith('')

    #extract body content using BeautifulSoup
    coverage_data = ''.join(['%s' % x for x in soup.body.contents])

    return coverage_data


def clinvar_submissions(store, user_id, institute_id):
    """Get all Clinvar submissions for a user and an institute"""
    submissions = list(store.clinvar_submissions(user_id, institute_id))
    return submissions


def clinvar_header(submission_objs, csv_type):
    """ Call clinvar parser to extract required fields to include in csv header from clinvar submission objects"""

    clinvar_header_obj = clinvar_submission_header(submission_objs, csv_type)
    return clinvar_header_obj


def clinvar_lines(clinvar_objects, clinvar_header):
    """ Call clinvar parser to extract required lines to include in csv file from clinvar submission objects and header"""

    clinvar_lines = clinvar_submission_lines(clinvar_objects, clinvar_header)
    return clinvar_lines


def update_synopsis(store, institute_obj, case_obj, user_obj, new_synopsis):
    """Update synopsis."""
    # create event only if synopsis was actually changed
    if new_synopsis and case_obj['synopsis'] != new_synopsis:
        link = url_for('cases.case', institute_id=institute_obj['_id'],
                       case_name=case_obj['display_name'])
        store.update_synopsis(institute_obj, case_obj, user_obj, link,
                              content=new_synopsis)


def hpo_diseases(username, password, hpo_ids, p_value_treshold=1):
    """Return the list of HGNC symbols that match annotated HPO terms.

    Args:
        username (str): username to use for phenomizer connection
        password (str): password to use for phenomizer connection

    Returns:
        query_result: a generator of dictionaries on the form
        {
            'p_value': float,
            'disease_source': str,
            'disease_nr': int,
            'gene_symbols': list(str),
            'description': str,
            'raw_line': str
        }
    """
    # skip querying Phenomizer unless at least one HPO terms exists
    try:
        results = query_phenomizer.query(username, password, *hpo_ids)
        diseases = [result for result in results
                    if result['p_value'] <= p_value_treshold]
        return diseases
    except SystemExit:
        return None


def rerun(store, mail, current_user, institute_id, case_name, sender, recipient):
    """Request a rerun by email."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    user_obj = store.user(current_user.email)
    link = url_for('cases.case', institute_id=institute_id, case_name=case_name)
    store.request_rerun(institute_obj, case_obj, user_obj, link)

    # this should send a JSON document to the SuSy API in the future
    html = """
        <p>{institute}: {case} ({case_id})</p>
        <p>Re-run requested by: {name}</p>
    """.format(institute=institute_obj['display_name'],
               case=case_obj['display_name'], case_id=case_obj['_id'],
               name=user_obj['name'].encode())

    # compose and send the email message
    msg = Message(subject=("SCOUT: request RERUN for {}"
                           .format(case_obj['display_name'])),
                  html=html, sender=sender, recipients=[recipient],
                  # cc the sender of the email for confirmation
                  cc=[user_obj['email']])
    mail.send(msg)


def update_default_panels(store, current_user, institute_id, case_name, panel_ids):
    """Update default panels for a case."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    user_obj = store.user(current_user.email)
    link = url_for('cases.case', institute_id=institute_id, case_name=case_name)
    panel_objs = [store.panel(panel_id) for panel_id in panel_ids]
    store.update_default_panels(institute_obj, case_obj, user_obj, link, panel_objs)

def vcf2cytosure(store, institute_id, case_name, individual_id):
    """vcf2cytosure CGH file for inidividual."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)

    for individual in case_obj['individuals']:
        if individual['individual_id'] == individual_id:
            individual_obj = individual

    return (individual_obj['display_name'], individual_obj['vcf2cytosure'])


def multiqc(store, institute_id, case_name):
    """Find MultiQC report for the case."""
    institute_obj, case_obj = institute_and_case(store, institute_id, case_name)
    return dict(
        institute=institute_obj,
        case=case_obj,
    )


def get_sanger_unevaluated(store, institute_id, user_id):
    """Get all variants for an institute having Sanger validations ordered but still not evaluated

        Args:
            store(scout.adapter.MongoAdapter)
            institute_id(str)

        Returns:
            unevaluated: a list that looks like this: [ {'case1': [varID_1, varID_2, .., varID_n]}, {'case2' : [varID_1, varID_2, .., varID_n]} ],
                         where the keys are case_ids and the values are lists of variants with Sanger ordered but not yet validated

    """

    # Retrieve a list of ids for variants with Sanger ordered grouped by case from the 'event' collection
    # This way is much faster than querying over all variants in all cases of an institute
    sanger_ordered_by_case = store.sanger_ordered(institute_id, user_id)
    unevaluated = []

    # for each object where key==case and value==[variant_id with Sanger ordered]
    for item in sanger_ordered_by_case:
        case_id = item['_id']
        # Get the case to collect display name
        case_obj = store.case(case_id=case_id)

        if not case_obj: # the case might have been removed
            continue

        case_display_name = case_obj.get('display_name')

        # List of variant document ids
        varid_list = item['vars']

        unevaluated_by_case = {}
        unevaluated_by_case[case_display_name] = []

        for var_id in varid_list:
            # For each variant with sanger validation ordered
            variant_obj = store.variant(document_id=var_id, case_id=case_id)

            # Double check that Sanger was ordered (and not canceled) for the variant
            if variant_obj is None or variant_obj.get('sanger_ordered') is None or variant_obj.get('sanger_ordered') is False:
                continue

            validation = variant_obj.get('validation', 'not_evaluated')

            # Check that the variant is not evaluated
            if validation in ['True positive', 'False positive']:
                continue

            unevaluated_by_case[case_display_name].append(variant_obj['_id'])

        # If for a case there is at least one Sanger validation to evaluate add the object to the unevaluated objects list
        if len(unevaluated_by_case[case_display_name]) > 0:
            unevaluated.append(unevaluated_by_case)

    return unevaluated



import logging

from pprint import pprint as pp

from scout.utils.md5 import generate_md5_key
from .genotype import parse_genotypes
from .compound import parse_compounds
from .clnsig import parse_clnsig
from .gene import parse_genes
from .frequency import (parse_frequencies, parse_sv_frequencies)
from .conservation import parse_conservations
from .ids import parse_ids
from .callers import parse_callers
from .rank_score import parse_rank_score
from .coordinates import parse_coordinates
from .models import parse_genetic_models
from .transcript import parse_transcripts
from .deleteriousness import parse_cadd

from scout.constants import CHR_PATTERN

from scout.exceptions import VcfError

logger = logging.getLogger(__name__)

def parse_variant(variant, case, variant_type='clinical',
                 rank_results_header=None, vep_header=None,
                 individual_positions=None, category=None):
    """Return a parsed variant

        Get all the necessary information to build a variant object

    Args:
        variant(cyvcf2.Variant)
        case(dict)
        variant_type(str): 'clinical' or 'research'
        rank_results_header(list)
        vep_header(list)
        individual_positions(dict): Explain what position each individual has
                                    in vcf
        category(str): 'snv', 'sv', 'str' or 'cancer'

    Returns:
        parsed_variant(dict): Parsed variant
    """
    # These are to display how the rank score is built
    rank_results_header = rank_results_header or []
    # Vep information
    vep_header = vep_header or []

    parsed_variant = {}

    # Create the ID for the variant
    case_id = case['_id']
    if '-' in case_id:
        logger.debug('internal case id detected')
        genmod_key = case['display_name']
    else:
        genmod_key = case['_id']

    chrom_match = CHR_PATTERN.match(variant.CHROM)
    chrom = chrom_match.group(2)

    # Builds a dictionary with the different ids that are used

    if variant.ALT:
        alt=variant.ALT[0]
    elif not variant.ALT and category == "str":
        alt='.'

    parsed_variant['ids'] = parse_ids(
        chrom=chrom,
        pos=variant.POS,
        ref=variant.REF,
        alt=alt,
        case_id=case_id,
        variant_type=variant_type
    )
    parsed_variant['case_id'] = case_id
    # type can be 'clinical' or 'research'
    parsed_variant['variant_type'] = variant_type
    # category is sv or snv
    # cyvcf2 knows if it is a sv, indel or snv variant
    if not category:
        category = variant.var_type
        if category == 'indel':
            category = 'snv'
        if category == 'snp':
            category = 'snv'

    parsed_variant['category'] = category

    ################# General information #################

    parsed_variant['reference'] = variant.REF

    ### We allways assume splitted and normalized vcfs!!!
    if len(variant.ALT) > 1:
        raise VcfError("Variants are only allowed to have one alternative")
    parsed_variant['alternative'] = alt

    # cyvcf2 will set QUAL to None if '.' in vcf
    parsed_variant['quality'] = variant.QUAL

    if variant.FILTER:
        parsed_variant['filters'] = variant.FILTER.split(';')
    else:
        parsed_variant['filters'] = ['PASS']

    # Add the dbsnp ids
    parsed_variant['dbsnp_id'] = variant.ID

    # This is the id of other position in translocations
    # (only for specific svs)
    parsed_variant['mate_id'] = None

    ################# Position specific #################
    parsed_variant['chromosome'] = chrom

    coordinates = parse_coordinates(variant, category)

    parsed_variant['position'] = coordinates['position']
    parsed_variant['sub_category'] = coordinates['sub_category']
    parsed_variant['mate_id'] = coordinates['mate_id']
    parsed_variant['end'] = coordinates['end']
    parsed_variant['length'] = coordinates['length']
    parsed_variant['end_chrom'] = coordinates['end_chrom']
    parsed_variant['cytoband_start'] = coordinates['cytoband_start']
    parsed_variant['cytoband_end'] = coordinates['cytoband_end']

    ################# Add rank score #################
    # The rank score is central for displaying variants in scout.

    rank_score = parse_rank_score(variant.INFO.get('RankScore', ''), genmod_key)
    parsed_variant['rank_score'] = rank_score or 0


    ################# Add gt calls #################
    if individual_positions and case['individuals']:
        parsed_variant['samples'] = parse_genotypes(variant, case['individuals'],
                                                    individual_positions)
    else:
        parsed_variant['samples'] = []

    ################# Add compound information #################
    compounds = parse_compounds(compound_info=variant.INFO.get('Compounds'),
                                case_id=genmod_key,
                                variant_type=variant_type)
    if compounds:
        parsed_variant['compounds'] = compounds

    ################# Add inheritance patterns #################

    genetic_models = parse_genetic_models(variant.INFO.get('GeneticModels'), genmod_key)
    if genetic_models:
        parsed_variant['genetic_models'] = genetic_models

    ################# Add autozygosity calls if present #################

    azlength = variant.INFO.get('AZLENGTH')
    if azlength:
        parsed_variant['azlength'] = int(azlength)

    azqual = variant.INFO.get('AZQUAL')
    if azqual:
        parsed_variant['azqual'] = float(azqual)

    ################ Add STR info if present ################

    # repeat id generally corresponds to gene symbol
    repeat_id = variant.INFO.get('REPID')
    if repeat_id:
        parsed_variant['str_repid'] = str(repeat_id)

    # repeat unit - used e g in PanelApp naming of STRs
    repeat_unit = variant.INFO.get('RU')
    if repeat_unit:
        parsed_variant['str_ru'] = str(repeat_unit)

    # repeat unit - used e g in PanelApp naming of STRs
    repeat_ref = variant.INFO.get('REF')
    if repeat_unit:
        parsed_variant['str_ref'] = int(repeat_ref)

    ################# Add gene and transcript information #################
    raw_transcripts = []
    if vep_header:
        vep_info = variant.INFO.get('CSQ')
        if vep_info:
            raw_transcripts = (dict(zip(vep_header, transcript_info.split('|')))
                               for transcript_info in vep_info.split(','))

    parsed_transcripts = []
    dbsnp_ids = set()
    cosmic_ids = set()
    for parsed_transcript in parse_transcripts(raw_transcripts, parsed_variant['alternative']):
        parsed_transcripts.append(parsed_transcript)
        for dbsnp in parsed_transcript.get('dbsnp', []):
            dbsnp_ids.add(dbsnp)
        for cosmic in parsed_transcript.get('cosmic', []):
            cosmic_ids.add(cosmic)

    # The COSMIC tag in INFO is added via VEP and/or bcftools annotate

    cosmic_tag = variant.INFO.get('COSMIC')
    if cosmic_tag:
        cosmic_ids.add(cosmic_tag[4:])

    if (dbsnp_ids and not parsed_variant['dbsnp_id']):
        parsed_variant['dbsnp_id'] = ';'.join(dbsnp_ids)

    if cosmic_ids:
        parsed_variant['cosmic_ids'] = list(cosmic_ids)

    gene_info = parse_genes(parsed_transcripts)

    parsed_variant['genes'] = gene_info

    hgnc_ids = set([])

    for gene in parsed_variant['genes']:
        hgnc_ids.add(gene['hgnc_id'])

    parsed_variant['hgnc_ids'] = list(hgnc_ids)

    ################# Add clinsig prediction #################
    if variant.INFO.get('CLNACC'):
        acc = variant.INFO.get('CLNACC')
    else:
        acc = variant.INFO.get('CLNVID')
    clnsig_predictions = parse_clnsig(
        acc=acc,
        sig=variant.INFO.get('CLNSIG'),
        revstat=variant.INFO.get('CLNREVSTAT'),
        transcripts=parsed_transcripts
        )

    if clnsig_predictions:
        parsed_variant['clnsig'] = clnsig_predictions

    ################# Add the frequencies #################
    frequencies = parse_frequencies(variant, parsed_transcripts)

    parsed_variant['frequencies'] = frequencies

    # parse out old local observation count
    local_obs_old = variant.INFO.get('Obs')
    if local_obs_old:
        parsed_variant['local_obs_old'] = int(local_obs_old)

    local_obs_hom_old = variant.INFO.get('Hom')
    if local_obs_hom_old:
        parsed_variant['local_obs_hom_old'] = int(local_obs_hom_old)

    ###################### Add severity predictions ######################
    cadd = parse_cadd(variant, parsed_transcripts)
    if cadd:
        parsed_variant['cadd_score'] = cadd

    spidex = variant.INFO.get('SPIDEX')
    if spidex:
        parsed_variant['spidex'] = float(spidex)

    ###################### Add conservation ######################

    parsed_variant['conservation'] = parse_conservations(variant)

    parsed_variant['callers'] = parse_callers(variant, category=category)

    rank_result = variant.INFO.get('RankResult')
    if rank_result:
        results = [int(i) for i in rank_result.split('|')]
        parsed_variant['rank_result'] = dict(zip(rank_results_header, results))

    ###################### Add SV specific annotations ######################
    sv_frequencies = parse_sv_frequencies(variant)
    for key in sv_frequencies:
        parsed_variant['frequencies'][key] = sv_frequencies[key]

    ###################### Add Cancer specific annotations ######################
    # MSK_MVL indicates if variants are in the MSK managed variant list
    # https://www.ncbi.nlm.nih.gov/pmc/articles/PMC5437632/
    mvl_tag = variant.INFO.get('MSK_MVL')
    if mvl_tag:
        parsed_variant['mvl_tag'] = True

    return parsed_variant



import logging

from scout.models import Institute

LOG = logging.getLogger(__name__)

def build_institute(internal_id, display_name, sanger_recipients=None,
                    coverage_cutoff=None, frequency_cutoff=None):
    """Build a institute object

    Args:
        internal_id(str)
        display_name(str)
        sanger_recipients(list(str)): List with email addresses

    Returns:
        institute_obj(scout.models.Institute)

    """

    LOG.info("Building institute %s with display name %s", internal_id,display_name)

    institute_obj = Institute(
        internal_id=internal_id, 
        display_name=display_name, 
        sanger_recipients=sanger_recipients,
        coverage_cutoff = coverage_cutoff,
        frequency_cutoff = frequency_cutoff
    )
    
    for key in list(institute_obj):
        if institute_obj[key] is None:
            institute_obj.pop(key)

    return institute_obj



CONSEQUENCE = (
    'deleterious',
    'deleterious_low_confidence',
    'probably_damaging',
    'possibly_damaging',
    'tolerated',
    'tolerated_low_confidence',
    'benign',
    'unknown'
)

CONSERVATION = ('NotConserved', 'Conserved')

FEATURE_TYPES = (
  'exonic',
  'splicing',
  'ncRNA_exonic',
  'intronic',
  'ncRNA',
  'upstream',
  '5UTR',
  '3UTR',
  'downstream',
  'TFBS',
  'regulatory_region',
  'genomic_feature',
  'intergenic_variant'
)

SV_TYPES = (
    'ins',
    'del',
    'dup',
    'cnv',
    'inv',
    'bnd'
)

GENETIC_MODELS = (
  ('AR_hom', 'Autosomal Recessive Homozygote'),
  ('AR_hom_dn', 'Autosomal Recessive Homozygote De Novo'),
  ('AR_comp', 'Autosomal Recessive Compound'),
  ('AR_comp_dn', 'Autosomal Recessive Compound De Novo'),
  ('AD', 'Autosomal Dominant'),
  ('AD_dn', 'Autosomal Dominant De Novo'),
  ('XR', 'X Linked Recessive'),
  ('XR_dn', 'X Linked Recessive De Novo'),
  ('XD', 'X Linked Dominant'),
  ('XD_dn', 'X Linked Dominant De Novo'),
)


VARIANT_CALL = (
    'Pass',
    'Filtered',
    'Not Found',
    'Not Used',
)

# Describe conversion between numerical SPIDEX values and human readable.
# Abs is not tractable in mongo query.
SPIDEX_HUMAN = {
    'low': { 'neg': [-1,0], 'pos': [0, 1]},
    'medium': { 'neg': [-2,-1], 'pos': [1,2]},
    'high': { 'neg': [-2,-float('inf')], 'pos': [2,float('inf')]}
}

SPIDEX_LEVELS = (
    'not_reported',
    'low',
    'medium',
    'high'
)

MANUAL_RANK_OPTIONS = {
    8: {
        'label': 'known pathogenic',
        'description': 'Previously known pathogenic in Clinvar Hgmd literature etc',
    },
    7: {
        'label': 'pathogenic',
        'description': ("Novel mutation but overlapping phenotype with known pathogenic, "
                        "no further experimental validation needed"),
    },
    6: {
        'label': 'novel validated pathogenic',
        'description': 'Novel mutation and validated experimentally',
    },
    5: {
        'label': 'pathogenic partial phenotype',
        'description': ("Pathogenic variant explains part of patients phenotype, but "
                        "not all symptoms"),
    },
    4: {
        'label': 'likely pathogenic',
        'description': 'Experimental validation required to prove causality',
    },
    3: {
        'label': 'possibly pathogenic',
        'description': 'Uncertain significance, but cannot disregard yet',
    },
    2: {
        'label': 'likely benign',
        'description': 'Uncertain significance, but can discard',
    },
    1: {
        'label': 'benign',
        'description': 'Does not cause phenotype',
    },
    0: {
        'label': 'other',
        'description': 'Phenotype not related to disease',
    },
}

DISMISS_VARIANT_OPTIONS = {
    2: {
        'label': 'Common public',
        'description': 'Too common in public databases.',
        'evidence': ['freq']
        },
    3: {
        'label': 'Common local',
        'description': 'Too common in local databases.',
        'evidence': ['freq']
        },
    5: {
        'label': 'Irrelevant phenotype',
        'description': 'Phenotype not relevant.',
        'evidence': ['OMIM']
        },
    7: {
        'label': 'Inconsistent inheritance pattern',
        'description': 'Inheritance pattern not relevant.',
        'evidence': ['OMIM', 'GT', 'inheritance_model']
        },
    11: {
        'label': 'No plausible compound',
        'description': 'No plausible compound - AR disease.',
        'evidence': ['Compounds']
        },
    13: {
        'label': 'Not in disease transcript',
        'description': 'Not in transcript relevant to disease.',
        'evidence': ['transcript']
        },
    17: {
        'label': 'Not in RefSeq transcript',
        'description':
            'Not in a RefSeq transcript - could not be determined relevant.',
        'evidence': ['transcript']
        },
    19: {
        'label': 'Splicing unaffected',
        'description': 'Does not appear to affect splicing.',
        'evidence': ['spidex']
        },
    23: {
        'label': 'Inherited from unaffected',
        'description': 'Inherited from an unaffected individual.',
        'evidence': ['GT', 'pedigree']
        },
    29: {
        'label': 'Technical issues',
        'description': 'Technical issues - likely false positive.',
        'evidence': ['GT', 'pileup']
        },
    31: {
        'label': 'No protein function',
        'description': 
        'Not likely to alter protein function - eg benign polyQ expansion.',
        'evidence': ['CADD', 'conservation']
        },
    37: {
        'label': 'Reputation benign',
        'description': 'Reputable source classified benign.',
        'evidence': ['clinvar']
        },
    41: {
        'label': 'Common variation type',
        'description': 
        'Found in a gene with much benign such (e.g. missense) variation.',
        'evidence': ['type']
        },
    43: {
        'label': 'Unstudied variation type',
        'description': 
        'In a gene where mainly other types of variation (e.g. repeat expansion) are established as pathologic.',
        'evidence': ['type']
        }
}



import logging

import click

from .panels import panels
from .users import users
from .institutes import institutes
from .diseases import diseases
from .hpo import hpo
from .whitelist import whitelist
from .aliases import aliases
from .individuals import individuals
from .index import index
from .intervals import intervals
from .collections import collections
from .transcripts import transcripts


LOG = logging.getLogger(__name__)


@click.group()
@click.pass_context
def view(context):
    """
    View objects from the database.
    """
    pass


view.add_command(panels)
view.add_command(users)
view.add_command(institutes)
view.add_command(diseases)
view.add_command(hpo)
view.add_command(whitelist)
view.add_command(aliases)
view.add_command(individuals)
view.add_command(index)
view.add_command(intervals)
view.add_command(collections)
view.add_command(transcripts)



from pprint import pprint as pp
import pytest
import logging
import datetime
import pymongo

from scout.constants import VERBS_MAP

logger = logging.getLogger(__name__)

def test_create_event(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a database without any events

    assert adapter.event_collection.find().count() == 0

    ## WHEN inserting a event
    verb = "status"
    adapter.create_event(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='a link',
        category='case',
        verb=verb,
        subject='a subject',
        level='specific'
    )

    # THEN assert that the event was added to the database

    adapter.event_collection.find().count() == 1
    res = adapter.event_collection.find_one()

    assert res['verb'] == verb

def test_assign(adapter, institute_obj, case_obj, user_obj):
    logger.info("Testing assign a user to a case")
    # GIVEN a populated database without events
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)

    assert adapter.event_collection.find().count() == 0

    ## WHEN assigning a user to a case
    link = 'assignlink'
    updated_case = adapter.assign(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link
    )
    # THEN the case should have the user assigned
    assert updated_case['assignees'] == [user_obj['_id']]
    # THEN an event should have been created
    assert adapter.event_collection.find().count() == 1

    event_obj = adapter.event_collection.find_one()
    assert event_obj['link'] == link

def test_unassign(adapter, institute_obj, case_obj, user_obj):
    # GIVEN an adapter to a database with one assigned case
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)

    link = 'assignlink'
    updated_case = adapter.assign(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link
    )

    assert updated_case['assignees'] == [user_obj['_id']]

    assert adapter.event_collection.find().count() == 1

    # WHEN unassigning a user from a case
    updated_case = adapter.unassign(
         institute=institute_obj,
         case=case_obj,
         user=user_obj,
         link='unassignlink'
    )

    # THEN two events should have been created
    assert adapter.event_collection.find().count() == 2

    # THEN the case should not be assigned
    assert updated_case.get('assignees') == []
    # THEN a unassign event should be created
    event = adapter.event_collection.find_one({'verb': 'unassign'})
    assert event['link'] == 'unassignlink'

def test_update_synopsis(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated database without events
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)
    assert adapter.event_collection.find().count() == 0

    # WHEN updating synopsis for a case
    link = 'synopsislink'
    synopsis = "The updated synopsis"
    updated_case = adapter.update_synopsis(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link,
        content=synopsis
    )
    # THEN the case should have the synopsis added
    assert updated_case['synopsis'] == synopsis

    # THEN an event for synopsis should have been added
    event = adapter.event_collection.find_one({'link': link})
    assert event['content'] == synopsis

def test_archive_case(adapter, institute_obj, case_obj, user_obj):
    logger.info("Set a case to archive status")
    ## GIVEN a populated database without events
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)

    assert adapter.event_collection.find().count() == 0

    ## WHEN setting a case in archive status
    link = 'archivelink'
    updated_case = adapter.archive_case(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link,
    )
    ## THEN the case status should be archived
    assert updated_case['status'] == 'archived'

    ## THEN a event for archiving should be added
    event = adapter.event_collection.find_one({'link': link})
    assert event['link'] == link

def test_open_research(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated database without events
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)
    assert adapter.event_collection.find().count() == 0

    case = adapter.case_collection.find_one({'_id': case_obj['_id']})
    assert case.get('research_requested', False) == False


    ## WHEN setting opening research for a case
    link = 'openresearchlink'
    updated_case = adapter.open_research(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link,
    )
    ## THEN research_requested should be True
    assert updated_case['research_requested'] == True

    ## THEN an event for opening research should be created
    event = adapter.event_collection.find_one({'link': link})
    assert event['link'] == 'openresearchlink'

def test_add_hpo(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated database without a gene and a hpo term
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)

    gene_obj = dict(
        hgnc_id = 1,
        hgnc_symbol = 'test',
        ensembl_id = 'anothertest',
        chromosome = '1',
        start = 10,
        end = 20,
        build = '37'
    )
    adapter.load_hgnc_gene(gene_obj)

    hpo_obj = dict(
        _id = 'HP:0000878',
        hpo_id = 'HP:0000878',
        description = 'A term',
        genes = [1]
    )

    adapter.load_hpo_term(hpo_obj)

    hpo_term = hpo_obj['hpo_id']

    ## WHEN adding a hpo term for a case
    link = 'addhpolink'

    updated_case = adapter.add_phenotype(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link,
        hpo_term=hpo_term,
    )
    ## THEN the case should have a hpo term
    for term in updated_case['phenotype_terms']:
        assert term['phenotype_id'] == hpo_term

    ## THEN a event should have been created
    event = adapter.event_collection.find_one({'link': link})
    assert event['link'] == link

def test_add_phenotype_group(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated database without a gene and a hpo term
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)

    gene_obj = dict(
        hgnc_id = 1,
        hgnc_symbol = 'test',
        ensembl_id = 'anothertest',
        chromosome = '1',
        start = 10,
        end = 20,
        build = '37'
    )
    adapter.load_hgnc_gene(gene_obj)

    hpo_obj = dict(
        _id = 'HP:0000878',
        hpo_id = 'HP:0000878',
        description = 'A term',
        genes = [1]
    )

    adapter.load_hpo_term(hpo_obj)

    hpo_term = hpo_obj['hpo_id']

    # GIVEN a populated database with no events
    assert adapter.hpo_term_collection.find().count() > 0
    assert adapter.hpo_term_collection.find({'_id':hpo_term})
    assert adapter.event_collection.find().count() == 0

    ## WHEN adding a phenotype group
    updated_case = adapter.add_phenotype(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='hpolink',
        hpo_term=hpo_term,
        is_group=True,
    )
    # THEN the case should have phenotypes
    assert len(updated_case['phenotype_terms']) > 0
    assert len(updated_case['phenotype_groups']) > 0

    # THEN there should be phenotype events
    assert adapter.event_collection.find().count() > 0

def test_add_wrong_hpo(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated database
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)
    # WHEN adding a non exoisting hpo term for a case
    hpo_term = 'k'
    with pytest.raises(ValueError):
    # THEN a value error should be raised
        adapter.add_phenotype(
            institute=institute_obj,
            case=case_obj,
            user=user_obj,
            link='addhpolink',
            hpo_term=hpo_term
        )

def test_add_no_term(adapter, institute_obj, case_obj, user_obj):
    logger.info("Add a HPO term for a case")
    ## GIVEN a populated database
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)
    ## WHEN adding hpo term without a term
    with pytest.raises(ValueError):
        ## THEN a value error should be raised
        adapter.add_phenotype(
            institute=institute_obj,
            case=case_obj,
            user=user_obj,
            link='addhpolink',
        )

def test_add_non_existing_mim(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated database
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)
    #Non existing mim phenotype
    mim_term = 'MIM:0000002'
    # GIVEN a populated database

    # WHEN adding a non existing phenotype term
    updated_case = adapter.add_phenotype(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='mimlink',
        omim_term=mim_term
    )
    # THEN the case should not have any phenotypes
    assert len(updated_case.get('phenotype_terms', [])) == 0

    # THEN there should not exist any events
    assert adapter.event_collection.find().count() == 0

def test_add_mim(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated adapter with a disease term
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)

    gene_obj = dict(
        hgnc_id = 1,
        hgnc_symbol = 'test',
        ensembl_id = 'anothertest',
        chromosome = '1',
        start = 10,
        end = 20,
        build = '37'
    )
    adapter.load_hgnc_gene(gene_obj)

    mim_obj = {
        "_id":"OMIM:605606",
        "disease_id":"OMIM:605606",
        "disease_nr":605606,
        "description":"Psoriasis susceptibility 7",
        "source":"OMIM",
        "genes":[1],
        "inheritance":[],
        "hpo_terms":['HP:0000878']
    }
    adapter.disease_term_collection.insert_one(mim_obj)

    hpo_obj = dict(
        _id = 'HP:0000878',
        hpo_id = 'HP:0000878',
        description = 'A term',
        genes = [1]
    )

    adapter.load_hpo_term(hpo_obj)

    #Existing mim phenotype
    mim_obj = adapter.disease_term_collection.find_one()
    mim_term = mim_obj['_id']

    assert adapter.hpo_term_collection.find().count() > 0
    # GIVEN a populated database
    assert adapter.event_collection.find().count() == 0

    # WHEN adding a existing phenotype term
    updated_case = adapter.add_phenotype(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='mimlink',
        omim_term=mim_term
    )
    # THEN the case should have phenotypes
    assert len(updated_case['phenotype_terms']) > 0

    # THEN there should be phenotype events
    assert adapter.event_collection.find().count() > 0

def test_remove_hpo(hpo_database, institute_obj, case_obj, user_obj):
    adapter = hpo_database
    logger.info("Add a HPO term for a case")
    adapter._add_case(case_obj)

    # GIVEN a populated database
    assert adapter.event_collection.find().count() == 0

    hpo_term = 'HP:0000878'

    updated_case = adapter.add_phenotype(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='addhpolink',
        hpo_term=hpo_term
    )

    #Assert that the term was added to the case
    assert len(updated_case['phenotype_terms']) == 1

    #Check that the event exists
    assert adapter.event_collection.find().count() == 1

    # WHEN removing the phenotype term
    updated_case = adapter.remove_phenotype(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link='removehpolink',
        phenotype_id=hpo_term
    )
    # THEN the case should not have phenotype terms
    assert len(updated_case['phenotype_terms']) == 0

    # THEN an event should have been created
    assert adapter.event_collection.find().count() == 2

def test_add_cohort(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated database
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)
    assert adapter.event_collection.find().count() == 0

    cohort_name = 'cohort'

    link = 'cohortlink'
    ## WHEN adding a cohort to a case
    updated_case = adapter.add_cohort(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link,
        tag=cohort_name
    )
    # THEN the case should have the cohort saved
    assert set(updated_case['cohorts']) == set([cohort_name])
    # THEN an event should have been created
    assert adapter.event_collection.find().count() == 1

    event_obj = adapter.event_collection.find_one()
    assert event_obj['link'] == link

def test_remove_cohort(adapter, institute_obj, case_obj, user_obj):
    ## GIVEN a populated database
    adapter.case_collection.insert_one(case_obj)
    adapter.institute_collection.insert_one(institute_obj)
    adapter.user_collection.insert_one(user_obj)
    assert adapter.event_collection.find().count() == 0

    cohort_name = 'cohort'

    link = 'cohortlink'
    ## WHEN adding a cohort to a case
    updated_case = adapter.add_cohort(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link,
        tag=cohort_name
    )
    assert adapter.event_collection.find().count() == 1

    remove_cohort_link = 'removeCohortlink'
    ## WHEN removing a cohort from a case
    updated_case = adapter.remove_cohort(
        institute=institute_obj,
        case=case_obj,
        user=user_obj,
        link=link,
        tag=cohort_name
    )
    # THEN the case should have the cohort saved
    assert updated_case['cohorts'] == []
    # THEN an event should have been created
    assert adapter.event_collection.find().count() == 2

def test_update_default_panels(case_database, institute_obj, case_obj, user_obj):
    adapter = case_database
    print('')
    # GIVEN a case with one gene panel
    assert len(case_obj['panels']) == 1

    for panel in case_obj['panels']:
        if panel['panel_name'] == 'panel1':
            assert panel['is_default'] == True
            print(panel)

    new_panel = {
        '_id': 'an_id',
        'panel_id': 'an_id',
        'panel_name': 'panel2',
        'display_name': 'Test panel2',
        'version': 1.0,
        'updated_at': datetime.datetime.now(),
        'nr_genes': 263,
        'is_default': False
    }
    case_obj = adapter.case_collection.find_one_and_update(
        {'_id':case_obj['_id']},
        {
            '$addToSet': {'panels': new_panel},
        },
        return_document = pymongo.ReturnDocument.AFTER
    )

    assert len(case_obj['panels']) == 2

    # WHEN updating the default panels

    updated_case = adapter.update_default_panels(
         institute_obj=institute_obj,
         case_obj=case_obj,
         user_obj=user_obj,
         link='update_default_link',
         panel_objs = [new_panel]
    )

    # THEN the the updated case should have panel1 as not default and panel2
    # as default
    for panel in updated_case['panels']:
        if panel['panel_name'] == 'panel1':
            assert panel['is_default'] == False
        elif panel['panel_name'] == 'panel2':
            assert panel['is_default'] == True

    # assert adapter.event_collection.find().count() == 2
    #
    # # THEN the case should not be assigned
    # assert updated_case.get('assignees') == []
    # # THEN a unassign event should be created
    # event = adapter.event_collection.find_one({'verb': 'unassign'})
    # assert event['link'] == 'unassignlink'



from scout.server.blueprints.variants.controllers import variant_verification, variants_export_header, variant_export_lines

def url_for(param, institute_id, case_name, variant_id):
    pass


def test_sanger_mail_sent(mock_mail, real_variant_database, institute_obj, case_obj, user_obj,
                          mock_sender):
    adapter = real_variant_database
    ## GIVEN we have a variant the we want to order sanger for
    variant_obj = adapter.variant_collection.find_one()
    variant_obj['hgnc_symbols'] = ''
    variant_obj['panels'] = ''

    ## WHEN calling variant_verification method with order==True
    variant_verification(adapter, mock_mail, institute_obj, case_obj, user_obj, variant_obj, mock_sender, 'complete_variant_url', True, url_builder=url_for)

    ## THEN the supplied mail objects send method should have been called
    assert mock_mail._send_was_called
    ## THEN the supplied mail objects send method should have received a message object
    assert mock_mail._message

def test_cancel_sanger_mail_sent(mock_mail, real_variant_database, institute_obj, case_obj, user_obj,
                                  mock_sender):
    adapter = real_variant_database
    ## GIVEN we have a variant the we want to order sanger for
    variant_obj = adapter.variant_collection.find_one()
    variant_obj['hgnc_symbols'] = ''
    variant_obj['panels'] = ''

    ## WHEN calling variant_verification method with order==False
    variant_verification(adapter, mock_mail, institute_obj, case_obj, user_obj, variant_obj, mock_sender, 'complete_variant_url', False, url_builder=url_for)

    ## THEN the supplied mail objects send method should have been called
    assert mock_mail._send_was_called
    ## THEN the supplied mail objects send method should have received a message object
    assert mock_mail._message

def test_variant_csv_export(real_variant_database, case_obj):
    adapter = real_variant_database
    case_id = case_obj['_id']

    # Given a database with variants from a case
    snv_variants = adapter.variant_collection.find({'case_id' : case_id, "category" : "snv"})

    # Given 5 variants to be exported
    variants_to_export = []
    for variant in snv_variants.limit(5):
        assert type(variant) is dict
        variants_to_export.append(variant)
    n_vars = len(variants_to_export)
    assert n_vars == 5

    # Collect export header from variants controller
    export_header = variants_export_header(case_obj)

    # Assert that exported document has n fields:
    # n = (EXPORT_HEADER items in variants_export.py) + (3 * number of individuals analysed for the case)
    assert len(export_header) == 8 + 3 * len(case_obj['individuals'])

    # Given the lines of the document to be exported
    export_lines = variant_export_lines(adapter, case_obj, variants_to_export)

    # Assert that all five variants are going to be exported to CSV
    assert len(export_lines) == 5

    # Assert that all of 5 variants contain the fields specified by the document header
    for export_line in export_lines:
        export_cols = export_line.split(',')
        assert len(export_cols) == len(export_header)



# -*- coding: utf-8 -*-
from datetime import datetime

panel_gene = dict(
    hgnc_id = int, # required
    symbol = str,

    disease_associated_transcripts = list, # list of strings that represent refseq transcripts
    reduced_penetrance = bool,
    mosaicism = bool,
    database_entry_version = str,
    inheritance_models = list, 
)


## gene panel should be indexed on genes.hgnc_id
gene_panel = dict(
    panel_name = str, # required
    institute = str, # institute_id
    version = float, # required
    date = datetime, # required
    display_name = str, # default is panel_name
    genes = list, # list of panel genes, sorted on panel_gene['symbol']
)



import logging

from pprint import pprint as pp

from pandas import DataFrame

LOG = logging.getLogger(__name__)


def parse_transcripts(transcript_lines):
    """Parse and massage the transcript information

    There could be multiple lines with information about the same transcript.
    This is why it is necessary to parse the transcripts first and then return a dictionary
    where all information has been merged.

    Args:
        transcript_lines(): This could be an iterable with strings or a pandas.DataFrame

    Returns:
        parsed_transcripts(dict): Map from enstid -> transcript info
    """
    LOG.info("Parsing transcripts")
    # Parse the transcripts, we need to check if it is a request or a file handle
    if isinstance(transcript_lines, DataFrame):
        transcripts = parse_ensembl_transcript_request(transcript_lines)
    else:
        transcripts = parse_ensembl_transcripts(transcript_lines)

    # Since there can be multiple lines with information about the same transcript
    # we store transcript information in a dictionary for now
    parsed_transcripts = {}
    # Loop over the parsed transcripts
    for tx in transcripts:
        tx_id = tx['ensembl_transcript_id']
        ens_gene_id = tx['ensembl_gene_id']

        # Check if the transcript has been added
        # If not, create a new transcript
        if not tx_id in parsed_transcripts:
            tx_info = {
                'chrom': tx['chrom'],
                'transcript_start': tx['transcript_start'],
                'transcript_end': tx['transcript_end'],
                'mrna': set(),
                'mrna_predicted': set(),
                'nc_rna': set(),
                'ensembl_gene_id': ens_gene_id,
                'ensembl_transcript_id': tx_id,
            }
            parsed_transcripts[tx_id] = tx_info
        
        tx_info = parsed_transcripts[tx_id]
        # Add the ref seq information
        if tx.get('refseq_mrna_predicted'):
            tx_info['mrna_predicted'].add(tx['refseq_mrna_predicted'])
        if tx.get('refseq_mrna'):
            tx_info['mrna'].add(tx['refseq_mrna'])
        if tx.get('refseq_ncrna'):
            tx_info['nc_rna'].add(tx['refseq_ncrna'])

    return parsed_transcripts


def parse_ensembl_gene_request(result):
    """Parse a dataframe with ensembl gene information

    Args:
        res(pandas.DataFrame)

    Yields:
        gene_info(dict)
    """
    LOG.info("Parsing genes from request")

    for index, row in result.iterrows():
        # print(index, row)
        ensembl_info = {}

        # Pandas represents missing data with nan which is a float
        if type(row['hgnc_symbol']) is float:
            # Skip genes without hgnc information
            continue

        ensembl_info['chrom'] = row['chromosome_name']
        ensembl_info['gene_start'] = int(row['start_position'])
        ensembl_info['gene_end'] = int(row['end_position'])
        ensembl_info['ensembl_gene_id'] = row['ensembl_gene_id']
        ensembl_info['hgnc_symbol'] = row['hgnc_symbol']

        hgnc_id = row['hgnc_id']

        if type(hgnc_id) is float:
            hgnc_id = int(hgnc_id)
        else:
            hgnc_id = int(hgnc_id.split(':')[-1])

        ensembl_info['hgnc_id'] = hgnc_id

        yield ensembl_info


def parse_ensembl_transcript_request(result):
    """Parse a dataframe with ensembl transcript information

    Args:
        res(pandas.DataFrame)

    Yields:
        transcript_info(dict)
    """
    LOG.info("Parsing transcripts from request")

    keys = [
        'chrom',
        'ensembl_gene_id',
        'ensembl_transcript_id',
        'transcript_start',
        'transcript_end',
        'refseq_mrna',
        'refseq_mrna_predicted',
        'refseq_ncrna',
    ]
    # for res in result.itertuples():
    for index, row in result.iterrows():
        ensembl_info = {}

        ensembl_info['chrom'] = str(row['chromosome_name'])
        ensembl_info['ensembl_gene_id'] = row['ensembl_gene_id']
        ensembl_info['ensembl_transcript_id'] = row['ensembl_transcript_id']

        ensembl_info['transcript_start'] = int(row['transcript_start'])
        ensembl_info['transcript_end'] = int(row['transcript_end'])

        # Check if refseq data is annotated
        # Pandas represent missing data with nan
        for key in keys[-3:]:
            if type(row[key]) is float:
                ensembl_info[key] = None
            else:
                ensembl_info[key] = row[key]
        yield ensembl_info


def parse_ensembl_line(line, header):
    """Parse an ensembl formated line

        Args:
            line(list): A list with ensembl gene info
            header(list): A list with the header info

        Returns:
            ensembl_info(dict): A dictionary with the relevant info
    """
    line = line.rstrip().split('\t')
    header = [head.lower() for head in header]
    raw_info = dict(zip(header, line))

    ensembl_info = {}

    for word in raw_info:
        value = raw_info[word]

        if not value:
            continue

        if 'chromosome' in word:
            ensembl_info['chrom'] = value

        if 'gene' in word:
            if 'id' in word:
                ensembl_info['ensembl_gene_id'] = value
            elif 'start' in word:
                ensembl_info['gene_start'] = int(value)
            elif 'end' in word:
                ensembl_info['gene_end'] = int(value)

        if 'hgnc symbol' in word:
            ensembl_info['hgnc_symbol'] = value
        if "gene name" in word:
            ensembl_info['hgnc_symbol'] = value

        if 'hgnc id' in word:
            ensembl_info['hgnc_id'] = int(value.split(':')[-1])

        if 'transcript' in word:
            if 'id' in word:
                ensembl_info['ensembl_transcript_id'] = value
            elif 'start' in word:
                ensembl_info['transcript_start'] = int(value)
            elif 'end' in word:
                ensembl_info['transcript_end'] = int(value)

        if 'exon' in word:
            if 'start' in word:
                ensembl_info['exon_start'] = int(value)
            elif 'end' in word:
                ensembl_info['exon_end'] = int(value)
            elif 'rank' in word:
                ensembl_info['exon_rank'] = int(value)

        if 'utr' in word:

            if 'start' in word:
                if '5' in word:
                    ensembl_info['utr_5_start'] = int(value)
                elif '3' in word:
                    ensembl_info['utr_3_start'] = int(value)
            elif 'end' in word:
                if '5' in word:
                    ensembl_info['utr_5_end'] = int(value)
                elif '3' in word:
                    ensembl_info['utr_3_end'] = int(value)

        if 'strand' in word:
            ensembl_info['strand'] = int(value)

        if 'refseq' in word:
            if 'mrna' in word:
                if 'predicted' in word:
                    ensembl_info['refseq_mrna_predicted'] = value
                else:
                    ensembl_info['refseq_mrna'] = value

            if 'ncrna' in word:
                ensembl_info['refseq_ncrna'] = value

    return ensembl_info


def parse_ensembl_genes(lines):
    """Parse lines with ensembl formated genes

        This is designed to take a biomart dump with genes from ensembl.
        Mandatory columns are:
        'Gene ID' 'Chromosome' 'Gene Start' 'Gene End' 'HGNC symbol

        Args:
            lines(iterable(str)): An iterable with ensembl formated genes
        Yields:
            ensembl_gene(dict): A dictionary with the relevant information
    """
    LOG.info("Parsing ensembl genes from file")
    header = []
    for index, line in enumerate(lines):

        # File allways start with a header line
        if index == 0:
            header = line.rstrip().split('\t')
            continue
        # After that each line represents a gene

        yield parse_ensembl_line(line, header)


def parse_ensembl_transcripts(lines):
    """Parse lines with ensembl formated transcripts

        This is designed to take a biomart dump with transcripts from ensembl.
        Mandatory columns are:
        'Gene ID' 'Transcript ID' 'Transcript Start' 'Transcript End' 'RefSeq mRNA'

        Args:
            lines(iterable(str)): An iterable with ensembl formated genes
        Yields:
            ensembl_gene(dict): A dictionary with the relevant information
    """
    header = []
    LOG.info("Parsing ensembl genes from file")
    for index, line in enumerate(lines):

        # File allways start with a header line
        if index == 0:
            header = line.rstrip().split('\t')
        # After that each line represents a transcript
        else:
            yield parse_ensembl_line(line, header)


def parse_ensembl_exons(lines):
    """Parse lines with ensembl formated exons

        This is designed to take a biomart dump with exons from ensembl.
        Check documentation for spec for download

        Args:
            lines(iterable(str)): An iterable with ensembl formated exons
        Yields:
            ensembl_gene(dict): A dictionary with the relevant information
    """
    header = []
    LOG.debug("Parsing ensembl exons...")
    for index, line in enumerate(lines):

        # File allways start with a header line
        if index == 0:
            header = line.rstrip().split('\t')
            continue

        exon_info = parse_ensembl_line(line, header)
        chrom = exon_info['chrom']
        start = exon_info['exon_start']
        end = exon_info['exon_end']
        transcript = exon_info['ensembl_transcript_id']
        gene = exon_info['ensembl_gene_id']

        rank = exon_info['exon_rank']
        strand = exon_info['strand']

        # Recalculate start and stop (taking UTR regions into account for end exons)
        if strand == 1:
            # highest position: start of exon or end of 5' UTR
            # If no 5' UTR make sure exon_start is allways choosen
            start = max(start, exon_info.get('utr_5_end') or -1)
            # lowest position: end of exon or start of 3' UTR
            end = min(end, exon_info.get('utr_3_start') or float('inf'))
        elif strand == -1:
            # highest position: start of exon or end of 3' UTR
            start = max(start, exon_info.get('utr_3_end') or -1)
            # lowest position: end of exon or start of 5' UTR
            end = min(end, exon_info.get('utr_5_start') or float('inf'))

        exon_id = "-".join([chrom, str(start), str(end)])

        if start > end:
            raise ValueError("ERROR: %s" % exon_id)
        data = {
            "exon_id": exon_id,
            "chrom": chrom,
            "start": start,
            "end": end,
            "transcript": transcript,
            "gene": gene,
            "rank": rank,
        }

        yield data


def parse_ensembl_exon_request(result):
    """Parse a dataframe with ensembl exon information

    Args:
        res(pandas.DataFrame)

    Yields:
        gene_info(dict)
    """
    keys = [
        'chrom',
        'gene',
        'transcript',
        'exon_id',
        'exon_chrom_start',
        'exon_chrom_end',
        '5_utr_start',
        '5_utr_end',
        '3_utr_start',
        '3_utr_end',
        'strand',
        'rank'
    ]

    # for res in result.itertuples():
    for res in zip(result['Chromosome/scaffold name'],
                   result['Gene stable ID'],
                   result['Transcript stable ID'],
                   result['Exon stable ID'],
                   result['Exon region start (bp)'],
                   result['Exon region end (bp)'],
                   result["5' UTR start"],
                   result["5' UTR end"],
                   result["3' UTR start"],
                   result["3' UTR end"],
                   result["Strand"],
                   result["Exon rank in transcript"]):
        ensembl_info = dict(zip(keys, res))

        # Recalculate start and stop (taking UTR regions into account for end exons)
        if ensembl_info['strand'] == 1:
            # highest position: start of exon or end of 5' UTR
            # If no 5' UTR make sure exon_start is allways choosen
            start = max(ensembl_info['exon_chrom_start'], ensembl_info['5_utr_end'] or -1)
            # lowest position: end of exon or start of 3' UTR
            end = min(ensembl_info['exon_chrom_end'], ensembl_info['3_utr_start'] or float('inf'))
        elif ensembl_info['strand'] == -1:
            # highest position: start of exon or end of 3' UTR
            start = max(ensembl_info['exon_chrom_start'], ensembl_info['3_utr_end'] or -1)
            # lowest position: end of exon or start of 5' UTR
            end = min(ensembl_info['exon_chrom_end'], ensembl_info['5_utr_start'] or float('inf'))

        ensembl_info['start'] = start
        ensembl_info['end'] = end

        yield ensembl_info

        # if type(ensembl_info['hgnc_symbol']) is float:
        #     # Skip genes without hgnc information
        #     continue
        # ensembl_info['gene_start'] = int(ensembl_info['gene_start'])
        # ensembl_info['gene_end'] = int(ensembl_info['gene_end'])
        #
        # if type(ensembl_info['hgnc_id']) is float:
        #     ensembl_info['hgnc_id'] = int(ensembl_info['hgnc_id'])
        # else:
        #     ensembl_info['hgnc_id'] = int(ensembl_info['hgnc_id'].split(':')[-1])
        #
        # yield ensembl_info



# -*- coding: utf-8 -*-
"""
scout.models.acmg_evaluation
~~~~~~~~~~~~~~~~~~

Define a document to describe a ACMG evaluation

Evaluations are stored in its own collection

"""

from datetime import datetime

evaluation = dict(
    variant_specific = str, #md5 document id
    variant_id = str, #md5 variant id
    institute_id = str, # Institute _id, required
    case_id = str, # case_id, required
    classification = str, # What did the evaluation end up in?

    # All evaluations will have an author
    user_id = str, # user email, required
    user_name = str, # user name
    criterias = list, # List of dictionaries with criterias
    # timestamps
    created_at = datetime,
)




# -*- coding: utf-8 -*-
import logging

from pprint import pprint as pp

from click import progressbar

from scout.build import build_hgnc_gene
from scout.utils.link import link_genes

from scout.utils.requests import (fetch_ensembl_genes, fetch_hgnc, fetch_mim_files,
                                 fetch_exac_constraint, fetch_hpo_files, 
                                 fetch_ensembl_transcripts, fetch_ensembl_exons)

from scout.load.transcript import load_transcripts

LOG = logging.getLogger(__name__)

def load_hgnc(adapter, genes=None, ensembl_lines=None, hgnc_lines=None, exac_lines=None, mim2gene_lines=None,
              genemap_lines=None, hpo_lines=None, transcripts_lines=None, build='37', omim_api_key=''):
    """Load Genes and transcripts into the database
              
    If no resources are provided the correct ones will be fetched.
    
    Args:
        adapter(scout.adapter.MongoAdapter)
        genes(dict): If genes are already parsed
        ensembl_lines(iterable(str)): Lines formated with ensembl gene information
        hgnc_lines(iterable(str)): Lines with gene information from genenames.org
        exac_lines(iterable(str)): Lines with information pLi-scores from ExAC
        mim2gene(iterable(str)): Lines with map from omim id to gene symbol
        genemap_lines(iterable(str)): Lines with information of omim entries
        hpo_lines(iterable(str)): Lines information about map from hpo terms to genes
        transcripts_lines(iterable): iterable with ensembl transcript lines
        build(str): What build to use. Defaults to '37'

    """
    gene_objs = load_hgnc_genes(
        adapter=adapter, 
        genes = genes, 
        ensembl_lines=ensembl_lines, 
        hgnc_lines=hgnc_lines, 
        exac_lines=exac_lines, 
        mim2gene_lines=mim2gene_lines,
        genemap_lines=genemap_lines, 
        hpo_lines=hpo_lines, 
        build=build, 
        omim_api_key=omim_api_key,
    )
    
    ensembl_genes = {}
    for gene_obj in gene_objs:
        ensembl_genes[gene_obj['ensembl_id']] = gene_obj
    
    transcript_objs = load_transcripts(
        adapter=adapter, 
        transcripts_lines=transcripts_lines, 
        build=build, 
        ensembl_genes=ensembl_genes)

def load_hgnc_genes(adapter, genes = None, ensembl_lines=None, hgnc_lines=None, exac_lines=None, mim2gene_lines=None,
                    genemap_lines=None, hpo_lines=None, build='37', omim_api_key=''):
    """Load genes into the database
        
    link_genes will collect information from all the different sources and 
    merge it into a dictionary with hgnc_id as key and gene information as values.

    Args:
        adapter(scout.adapter.MongoAdapter)
        genes(dict): If genes are already parsed
        ensembl_lines(iterable(str)): Lines formated with ensembl gene information
        hgnc_lines(iterable(str)): Lines with gene information from genenames.org
        exac_lines(iterable(str)): Lines with information pLi-scores from ExAC
        mim2gene(iterable(str)): Lines with map from omim id to gene symbol
        genemap_lines(iterable(str)): Lines with information of omim entries
        hpo_lines(iterable(str)): Lines information about map from hpo terms to genes
        build(str): What build to use. Defaults to '37'

    Returns:
        gene_objects(list): A list with all gene_objects that was loaded into database
    """
    gene_objects = list()
    
    if not genes:
        # Fetch the resources if not provided
        if ensembl_lines is None:
            ensembl_lines = fetch_ensembl_genes(build=build)
        hgnc_lines = hgnc_lines or fetch_hgnc()
        exac_lines = exac_lines or fetch_exac_constraint()
        if not (mim2gene_lines and genemap_lines):
            if not omim_api_key:
                raise SyntaxError("Need to provide omim api key")
            mim_files = fetch_mim_files(omim_api_key, mim2genes=True, genemap2=True)
            mim2gene_lines = mim_files['mim2genes']
            genemap_lines = mim_files['genemap2']
        if not hpo_lines:
            hpo_files = fetch_hpo_files(hpogenes=True)
            hpo_lines = hpo_files['hpogenes']
        
        
        # Link the resources
        genes = link_genes(
            ensembl_lines=ensembl_lines,
            hgnc_lines=hgnc_lines,
            exac_lines=exac_lines,
            mim2gene_lines=mim2gene_lines,
            genemap_lines=genemap_lines,
            hpo_lines=hpo_lines
        )

    non_existing = 0
    nr_genes = len(genes)
    
    with progressbar(genes.values(), label="Building genes", length=nr_genes) as bar:
        for gene_data in bar:
            if not gene_data.get('chromosome'):
                LOG.debug("skipping gene: %s. No coordinates found", gene_data.get('hgnc_symbol', '?'))
                non_existing += 1
                continue
        
            gene_obj = build_hgnc_gene(gene_data, build=build)
            gene_objects.append(gene_obj)

    LOG.info("Loading genes build %s", build)
    adapter.load_hgnc_bulk(gene_objects)

    LOG.info("Loading done. %s genes loaded", len(gene_objects))
    LOG.info("Nr of genes without coordinates in build %s: %s", build,non_existing)
    
    return gene_objects



import logging
import click

LOG = logging.getLogger(__name__)


@click.command('institutes', short_help='Display institutes')
@click.option('-i', '--institute-id',
    help="What institute to show"
)
@click.option('--json',
    help="Show json format",
    is_flag=True
)
@click.pass_context
def institutes(context, institute_id, json):
    """Show all institutes in the database"""
    LOG.info("Running scout view institutes")
    adapter = context.obj['adapter']

    if institute_id:
        institute_objs = []
        institute_obj = adapter.institute(institute_id)
        if not institute_obj:
            LOG.info("Institute %s does not exost", institute_id)
            return
        institute_objs.append(institute_obj)
    else:
        institute_objs = [ins_obj for ins_obj in adapter.institutes()]
    
    if len(institute_objs) == 0:
        click.echo("No institutes found")
        context.abort()

    header = ''
    if not json:
        for key in institute_objs[0].keys():
            header = header + "{0}\t".format(key)
        
        click.echo(header)

    for institute_obj in institute_objs:
        if json:
            click.echo(institute_obj)
            continue

        row = ''
        for value in institute_obj.values():
            row = row + "{0}\t".format(value)

        click.echo(row)



# -*- coding: utf-8 -*-
import logging

from . import (build_genotype, build_compound, build_gene, build_clnsig)

LOG = logging.getLogger(__name__)

def build_variant(variant, institute_id, gene_to_panels = None,
                  hgncid_to_gene=None, sample_info=None):
    """Build a variant object based on parsed information

        Args:
            variant(dict)
            institute_id(str)
            gene_to_panels(dict): A dictionary with
                {<hgnc_id>: {
                    'panel_names': [<panel_name>, ..],
                    'disease_associated_transcripts': [<transcript_id>, ..]
                    }
                    .
                    .
                }

            hgncid_to_gene(dict): A dictionary with
                {<hgnc_id>: <hgnc_gene info>
                    .
                    .
                }
            sample_info(dict): A dictionary with info about samples.
                               Strictly for cancer to tell which is tumor


        Returns:
            variant_obj(dict)

        variant = dict(
            # document_id is a md5 string created by institute_genelist_caseid_variantid:
            _id = str, # required, same as document_id
            document_id = str, # required
            # variant_id is a md5 string created by chrom_pos_ref_alt (simple_id)
            variant_id = str, # required
            # display name is variant_id (no md5)
            display_name = str, # required

            # chrom_pos_ref_alt
            simple_id = str,
            # The variant can be either research or clinical.
            # For research variants we display all the available information while
            # the clinical variants have limited annotation fields.
            variant_type = str, # required, choices=('research', 'clinical'))

            category = str, # choices=('sv', 'snv')
            sub_category = str, # choices=('snv', 'indel', 'del', 'ins', 'dup', 'inv', 'cnv', 'bnd')
            mate_id = str, # For SVs this identifies the other end

            case_id = str, # case_id is a string like owner_caseid
            chromosome = str, # required
            position = int, # required
            end = int, # required
            length = int, # required
            reference = str, # required
            alternative = str, # required

            rank_score = float, # required
            variant_rank = int, # required
            rank_score_results = list, # List if dictionaries
            variant_rank = int, # required

            institute = str, # institute_id, required

            sanger_ordered = bool,
            validation = str, # Sanger validation, choices=('True positive', 'False positive')

            quality = float,
            filters = list, # list of strings
            samples = list, # list of dictionaries that are <gt_calls>
            genetic_models = list, # list of strings choices=GENETIC_MODELS
            compounds = list, # sorted list of <compound> ordering='combined_score'

            genes = list, # list with <gene>
            dbsnp_id = str,

            # Gene ids:
            hgnc_ids = list, # list of hgnc ids (int)
            hgnc_symbols = list, # list of hgnc symbols (str)
            panels = list, # list of panel names that the variant ovelapps

            # Frequencies:
            thousand_genomes_frequency = float,
            thousand_genomes_frequency_left = float,
            thousand_genomes_frequency_right = float,
            exac_frequency = float,
            max_thousand_genomes_frequency = float,
            max_exac_frequency = float,
            local_frequency = float,
            local_obs_old = int,
            local_obs_hom_old = int,
            local_obs_total_old = int, # default=638
            # Predicted deleteriousness:
            cadd_score = float,
            clnsig = list, # list of <clinsig>
            spidex = float,

            missing_data = bool, # default False

            # Callers
            gatk = str, # choices=VARIANT_CALL, default='Not Used'
            samtools = str, # choices=VARIANT_CALL, default='Not Used'
            freebayes = str, # choices=VARIANT_CALL, default='Not Used'

            # Conservation:
            phast_conservation = list, # list of str, choices=CONSERVATION
            gerp_conservation = list, # list of str, choices=CONSERVATION
            phylop_conservation = list, # list of str, choices=CONSERVATION
            # Database options:
            gene_lists = list,
            manual_rank = int, # choices=[0, 1, 2, 3, 4, 5]
            dismiss_variant = list,

            acmg_evaluation = str, # choices=ACMG_TERMS
        )

    """
    gene_to_panels = gene_to_panels or {}
    hgncid_to_gene = hgncid_to_gene or {}
    sample_info = sample_info or {}

    #LOG.debug("Building variant %s", variant['ids']['document_id'])
    variant_obj = dict(
        _id = variant['ids']['document_id'],
        document_id=variant['ids']['document_id'],
        variant_id=variant['ids']['variant_id'],
        display_name=variant['ids']['display_name'],
        variant_type=variant['variant_type'],
        case_id=variant['case_id'],
        chromosome=variant['chromosome'],
        reference=variant['reference'],
        alternative=variant['alternative'],
        institute=institute_id,
    )

    variant_obj['missing_data'] = False
    variant_obj['position'] = int(variant['position'])
    variant_obj['rank_score'] = float(variant['rank_score'])

    end = variant.get('end')
    if end:
        variant_obj['end'] = int(end)

    length = variant.get('length')
    if length:
        variant_obj['length'] = int(length)

    variant_obj['simple_id'] = variant['ids'].get('simple_id')

    variant_obj['quality'] = float(variant['quality']) if variant['quality'] else None
    variant_obj['filters'] = variant['filters']

    variant_obj['dbsnp_id'] = variant.get('dbsnp_id')
    variant_obj['cosmic_ids'] = variant.get('cosmic_ids')

    variant_obj['category'] = variant['category']
    variant_obj['sub_category'] = variant.get('sub_category')

    if 'mate_id' in variant:
        variant_obj['mate_id'] = variant['mate_id']

    if 'cytoband_start' in variant:
        variant_obj['cytoband_start'] = variant['cytoband_start']

    if 'cytoband_end' in variant:
        variant_obj['cytoband_end'] = variant['cytoband_end']

    if 'end_chrom' in variant:
        variant_obj['end_chrom'] = variant['end_chrom']

    if 'str_ru' in variant:
        variant_obj['str_ru'] = variant['str_ru']

    if 'str_repid' in variant:
        variant_obj['str_repid'] = variant['str_repid']

    if 'str_ref' in variant:
        variant_obj['str_ref'] = variant['str_ref']

    gt_types = []
    for sample in variant.get('samples', []):
        gt_call = build_genotype(sample)
        gt_types.append(gt_call)

        if sample_info:
            sample_id = sample['individual_id']
            if sample_info[sample_id] == 'case':
                key = 'tumor'
            else:
                key = 'normal'
            variant_obj[key] = {
                    'alt_depth': sample['alt_depth'],
                    'ref_depth': sample['ref_depth'],
                    'read_depth': sample['read_depth'],
                    'alt_freq': sample['alt_frequency'],
                    'ind_id': sample_id
                }


    variant_obj['samples'] = gt_types

    if 'genetic_models' in variant:
        variant_obj['genetic_models'] = variant['genetic_models']

    # Add the compounds
    compounds = []
    for compound in variant.get('compounds', []):
        compound_obj = build_compound(compound)
        compounds.append(compound_obj)

    if compounds:
        variant_obj['compounds'] = compounds

    # Add the genes with transcripts
    genes = []
    for index, gene in enumerate(variant.get('genes', [])):
        if gene.get('hgnc_id'):
            gene_obj = build_gene(gene, hgncid_to_gene)
            genes.append(gene_obj)
            if index > 30:
                # avoid uploading too much data (specifically for SV variants)
                # mark variant as missing data
                variant_obj['missing_data'] = True
                break

    if genes:
        variant_obj['genes'] = genes

    # To make gene searches more effective
    if 'hgnc_ids' in variant:
        variant_obj['hgnc_ids'] = [hgnc_id for hgnc_id in variant['hgnc_ids'] if hgnc_id]

    # Add the hgnc symbols from the database genes
    hgnc_symbols = []
    for hgnc_id in variant_obj['hgnc_ids']:
        gene_obj = hgncid_to_gene.get(hgnc_id)
        if gene_obj:
            hgnc_symbols.append(gene_obj['hgnc_symbol'])
        # else:
            # LOG.warn("missing HGNC symbol for: %s", hgnc_id)

    if hgnc_symbols:
        variant_obj['hgnc_symbols'] = hgnc_symbols

    # link gene panels
    panel_names = set()
    for hgnc_id in variant_obj['hgnc_ids']:
        gene_panels = gene_to_panels.get(hgnc_id, set())
        panel_names = panel_names.union(gene_panels)

    if panel_names:
        variant_obj['panels'] = list(panel_names)

    # Add the clnsig ocbjects
    clnsig_objects = []
    for entry in variant.get('clnsig', []):
        clnsig_obj = build_clnsig(entry)
        clnsig_objects.append(clnsig_obj)

    if clnsig_objects:
        variant_obj['clnsig'] = clnsig_objects

    # Add the callers
    call_info = variant.get('callers', {})

    for caller in call_info:
        if call_info[caller]:
            variant_obj[caller] = call_info[caller]

    # Add the conservation
    conservation_info = variant.get('conservation', {})
    if conservation_info.get('phast'):
        variant_obj['phast_conservation'] = conservation_info['phast']

    if conservation_info.get('gerp'):
        variant_obj['gerp_conservation'] = conservation_info['gerp']

    if conservation_info.get('phylop'):
        variant_obj['phylop_conservation'] = conservation_info['phylop']

    # Add autozygosity calls
    if variant.get('azlength'):
        variant_obj['azlength'] = variant['azlength']

    if variant.get('azqual'):
        variant_obj['azqual'] = variant['azqual']

    # Add the frequencies
    frequencies = variant.get('frequencies', {})
    if frequencies.get('thousand_g'):
        variant_obj['thousand_genomes_frequency'] = float(frequencies['thousand_g'])

    if frequencies.get('thousand_g_max'):
        variant_obj['max_thousand_genomes_frequency'] = float(frequencies['thousand_g_max'])

    if frequencies.get('exac'):
        variant_obj['exac_frequency'] = float(frequencies['exac'])

    if frequencies.get('exac_max'):
        variant_obj['max_exac_frequency'] = float(frequencies['exac_max'])

    if frequencies.get('gnomad'):
        variant_obj['gnomad_frequency'] = float(frequencies['gnomad'])

    if frequencies.get('gnomad_max'):
        variant_obj['max_gnomad_frequency'] = float(frequencies['gnomad_max'])

    if frequencies.get('thousand_g_left'):
        variant_obj['thousand_genomes_frequency_left'] = float(frequencies['thousand_g_left'])

    if frequencies.get('thousand_g_right'):
        variant_obj['thousand_genomes_frequency_right'] = float(frequencies['thousand_g_right'])

    # add the local observation counts from the old archive
    if variant.get('local_obs_old'):
        variant_obj['local_obs_old'] =  variant['local_obs_old']

    if variant.get('local_obs_hom_old'):
        variant_obj['local_obs_hom_old'] = variant['local_obs_hom_old']

    # Add the sv counts:
    if frequencies.get('clingen_cgh_benign'):
        variant_obj['clingen_cgh_benign'] = frequencies['clingen_cgh_benign']
    if frequencies.get('clingen_cgh_pathogenic'):
        variant_obj['clingen_cgh_pathogenic'] = frequencies['clingen_cgh_pathogenic']
    if frequencies.get('clingen_ngi'):
        variant_obj['clingen_ngi'] = frequencies['clingen_ngi']
    if frequencies.get('decipher'):
        variant_obj['decipher'] = frequencies['decipher']

    # Add the severity predictors

    if variant.get('cadd_score'):
        variant_obj['cadd_score'] = variant['cadd_score']

    if variant.get('spidex'):
        variant_obj['spidex'] = variant['spidex']

    # Add the rank score results
    rank_results = []
    for category in variant.get('rank_result', []):
        rank_result = {
            'category': category,
            'score': variant['rank_result'][category]
        }
        rank_results.append(rank_result)

    if rank_results:
        variant_obj['rank_score_results'] = rank_results

    # Cancer specific
    if variant.get('mvl_tag'):
        variant_obj['mvl_tag'] = True

    return variant_obj



# -*- coding: utf-8 -*-
from pprint import pprint as pp

from scout.server.links import (add_gene_links, add_tx_links, omim)

def gene(store, hgnc_id):
    """Parse information about a gene."""
    res = {'builds': {'37': None, '38': None}, 'symbol': None, 'description': None, 'ensembl_id': None, 'record': None}

    for build in res['builds']:
        record = store.hgnc_gene(hgnc_id, build=build)
        if record:

            record['position'] = "{this[chromosome]}:{this[start]}-{this[end]}".format(this=record)
            res['aliases'] = record['aliases']
            res['hgnc_id'] = record['hgnc_id']
            res['description'] = record['description']
            res['builds'][build] = record
            res['symbol'] = record['hgnc_symbol']
            res['description'] = record['description']
            res['entrez_id'] = record.get('entrez_id')
            res['pli_score'] = record.get('pli_score')

            add_gene_links(record, int(build))

            res['omim_id'] = record.get('omim_id')
            res['incomplete_penetrance'] = record.get('incomplete_penetrance',False)
            res['inheritance_models'] = record.get('inheritance_models',[])
            for transcript in record['transcripts']:
                transcript['position'] = ("{this[chrom]}:{this[start]}-{this[end]}"
                                          .format(this=transcript))
                add_tx_links(transcript, build)

            for phenotype in record.get('phenotypes',[]):
                phenotype['omim_link'] = omim(phenotype.get('mim_number'))

            if not res['record']:
                res['record'] = record

    # If none of the genes where found
    if not any(res.values()):
        raise ValueError

    return res



def genes_to_json(store, query):
    """Fetch matching genes and convert to JSON."""
    gene_query = store.hgnc_genes(query, search=True)
    json_terms = [{'name': "{} | {} ({})".format(gene['hgnc_id'], gene['hgnc_symbol'],
                                                 ', '.join(gene['aliases'])),
                   'id': gene['hgnc_id']} for gene in gene_query]
    return json_terms



import click
import logging

from bson.json_util import dumps

from scout.export.variant import export_variants
from .utils import json_option

LOG = logging.getLogger(__name__)

@click.command('variants', short_help='Export variants')
@click.option('-c', '--collaborator',
        help="Specify what collaborator to export variants from. Defaults to cust000",
)
@click.option('-d', '--document-id',
        help="Search for a specific variant",
)
@json_option
@click.pass_context
def variants(context, collaborator, document_id, json):
    """Export causatives for a collaborator in .vcf format"""
    LOG.info("Running scout export variants")
    adapter = context.obj['adapter']
    collaborator = collaborator or 'cust000'
    
    variants = export_variants(adapter, collaborator, document_id=document_id)
    
    if json:
        click.echo(dumps([var for var in variants]))
        return
    
    vcf_header = [
        "##fileformat=VCFv4.2",
        "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO"
    ]

    for line in vcf_header:
        click.echo(line)

    #put variants in a dict to get unique ones
    variant_string = ("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}")

    for variant_obj in variants:
        click.echo(variant_string.format(
            variant_obj['chromosome'],
            variant_obj['position'],
            variant_obj['dbsnp_id'],
            variant_obj['reference'],
            variant_obj['alternative'],
            variant_obj['quality'],
            ';'.join(variant_obj['filters']),
            '.'
        ))


import pytest

from pprint import pprint as pp
from scout.parse.case import (parse_case, parse_ped, parse_individuals,
                              parse_individual, parse_case_data)
from scout.exceptions import PedigreeError
from scout.constants import REV_SEX_MAP

def test_parse_case(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case should have a owner
    assert case_data['owner'] == scout_config['owner']


def test_parse_case(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case a correct case id
    assert case_data['case_id'] == scout_config['family']


def test_parse_case_madeline(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case a correct case id
    assert case_data['madeline_info']


def test_parse_case_collaborators(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case should have a list with collaborators
    assert case_data['collaborators'] == [scout_config['owner']]


def test_parse_case_gene_panels(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case should have the same panels like the config
    assert case_data['gene_panels'] == scout_config['gene_panels']


def test_parse_case_default_panels(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case should have the same panels like the config
    assert case_data['default_panels'] == scout_config['default_gene_panels']


def test_parse_case_rank_threshold(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case should have the same panels like the config
    assert case_data['rank_score_threshold'] == scout_config['rank_score_threshold']


def test_parse_case_rank_model_version(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case should have the same rank model version like the config
    assert case_data['rank_model_version'] == scout_config['rank_model_version']


def test_parse_case_vcf_files(scout_config):
    # GIVEN you load sample information from a scout config
    # WHEN case is parsed
    case_data = parse_case(scout_config)
    # THEN the case should the same vcf files as specified in the
    assert case_data['vcf_files']['vcf_snv'] == scout_config['vcf_snv']
    assert case_data['vcf_files']['vcf_sv'] == scout_config['vcf_sv']
    assert case_data['vcf_files']['vcf_snv_research'] == scout_config['vcf_snv_research']
    assert case_data['vcf_files']['vcf_sv_research'] == scout_config['vcf_sv_research']


def test_parse_case_delivery_report(scout_config):
    # GIVEN you load sample information from a scout config

    # WHEN case is parsed
    case_data = parse_case(scout_config)

    # then we should find the delivery report in the parsed data
    assert case_data['delivery_report'] == scout_config['delivery_report']


def test_parse_ped_file(ped_file):
    # GIVEN a pedigree with three samples
    with open(ped_file, 'r') as case_lines:
        # WHEN parsing out relevant sample info
        family_id, samples = parse_ped(case_lines)
    # THEN it should return correct family id
    assert family_id == '643594'
    # THEN it should return correct number of individuals
    assert len(samples) == 3
    # THEN assert the sex has been converted
    for sample in samples:
        assert sample['sex'] in REV_SEX_MAP

def test_parse_case_ped_file(ped_file):
    # GIVEN a pedigree with three samples
    with open(ped_file, 'r') as case_lines:
        # WHEN parsing out relevant sample info
        config_data = parse_case_data(ped=case_lines, owner='cust000')
    # THEN it should return correct family id
    assert config_data['family'] == '643594'
    # THEN it should return correct number of individuals
    assert len(config_data['samples']) == 3


def test_parse_case_minimal_config(minimal_config):
    # GIVEN a minimal config
    # WHEN parsing the config
    case_data = parse_case(minimal_config)
    # THEN assert is was parsed correct
    assert case_data


def test_parse_ped():
    # GIVEN a pedigree with three samples
    case_lines = [
        "#Family ID\tIndividual ID\tPaternal ID\tMaternal ID\tSex\tPhenotype",
        "636808\tADM1059A1\t0\t0\t1\t1",
        "636808\tADM1059A2\tADM1059A1\tADM1059A3\t1\t2",
        "636808\tADM1059A3\t0\t0\t2\t1",
    ]

    # WHEN parsing out relevant sample info
    family_id, samples = parse_ped(case_lines)
    # THEN it should return correct family id
    assert family_id == '636808'
    # THEN it should return correct number of individuals
    assert len(samples) == 3


# ## Test how problems are handeled when parsing a case

def test_parse_case_two_cases_ped():
    # GIVEN ped lines from multiple families
    case_lines = [
        "#Family ID\tIndividual ID\tPaternal ID\tMaternal ID\tSex\tPhenotype",
        "636808\tADM1059A1\t0\t0\t1\t1",
        "636808\tADM1059A2\tADM1059A1\tADM1059A3\t1\t2",
        "636808\tADM1059A3\t0\t0\t2\t1",
        "636809\tADM1059A3\t0\t0\t2\t1",
    ]
    # WHEN parsing case info
    with pytest.raises(PedigreeError):
        # THEN it should raise since there are multiple families
        parse_ped(case_lines)


def test_no_individuals():
    # GIVEN a list with no indioviduals
    samples = []
    # WHEN parsing the individuals
    with pytest.raises(PedigreeError):
        # THEN error should be raised since a family has to have individuals
        parse_individuals(samples)


def test_parse_missing_id():
    # GIVEN a individual without sample_id
    sample_info = {
        'sex': 'male',
        'phenotype': 'affected',
    }
    # WHEN a individual is parsed
    with pytest.raises(PedigreeError):
        # THEN a PedigreeError should be raised
        parse_individual(sample_info)


def test_parse_missing_sex():
    # GIVEN a individual without sex
    sample_info = {
        'sample_id': '1',
        'phenotype': 'affected',
    }
    # WHEN a individual is parsed
    with pytest.raises(PedigreeError):
        # THEN a PedigreeError should be raised
        parse_individual(sample_info)


def test_parse_missing_phenotype():
    # GIVEN a individual without phenotype
    sample_info = {
        'sample_id': '1',
        'sex': 'male',
    }
    # WHEN a individual is parsed
    with pytest.raises(PedigreeError):
        # THEN a PedigreeError should be raised
        parse_individual(sample_info)


def test_parse_wrong_phenotype():
    # GIVEN a individual with wrong phenotype format
    sample_info = {
        'sample_id': '1',
        'sex': 'male',
        'phenotype': 'not-affected',
    }
    # WHEN a individual is parsed
    with pytest.raises(PedigreeError):
        # THEN a PedigreeError should be raised
        parse_individual(sample_info)


def test_parse_wrong_sex():
    # GIVEN a individual with wrong sex format
    sample_info = {
        'sample_id': '1',
        'sex': 'flale',
        'phenotype': 'affected',
    }
    # WHEN a individual is parsed
    with pytest.raises(PedigreeError):
        # THEN a PedigreeError should be raised
        parse_individual(sample_info)


def test_wrong_relations():
    """docstring for test_wrong_relations"""
    # GIVEN a individual with correct family info
    sample_info = {
        'sample_id': '1',
        'sex': 'male',
        'phenotype': 'affected',
        'mother': '3',
        'father': '2'
    }
    mother_info = {
        'sample_id': '3',
        'sex': 'female',
        'phenotype': 'unaffected',
        'mother': '0',
        'father': '0'
    }
    father_info = {
        'sample_id': '2',
        'sex': 'male',
        'phenotype': 'unaffected',
        'mother': '0',
        'father': '0'
    }
    samples = [sample_info, mother_info, father_info]
    # Nothong should happend here
    assert parse_individuals(samples)

    # WHEN changing mother id in proband
    sample_info['mother'] = '5'
    # THEN a PedigreeError should be raised
    with pytest.raises(PedigreeError):
        parse_individuals(samples)



from scout.parse.hgnc import (parse_hgnc_line,parse_hgnc_genes)

def test_parse_hgnc_line(hgnc_handle):
    """Test to parse a hgnc gene line"""
    header = next(hgnc_handle).split('\t')
    first_gene = next(hgnc_handle)
    gene_info = parse_hgnc_line(header=header, line=first_gene)
    assert gene_info['hgnc_id'] == int(first_gene.split('\t')[0].split(':')[1])

def test_parse_hgnc_genes(hgnc_handle):
    """Test to parse the hgnc genes"""
    genes = parse_hgnc_genes(lines=hgnc_handle)
    for gene in genes:
        if gene:
            assert gene['hgnc_id']



def test_load_case(case_obj, panel_database):
    adapter = panel_database

    # GIVEN a database with institute, user, genes, panel but no cases
    assert adapter.gene_panels().count() > 0
    assert adapter.users().count() > 0
    assert adapter.institutes().count() > 0

    # WHEN loading a case
    adapter._add_case(case_obj)

    # THEN assert that the case have been loaded with correct info
    assert adapter.cases().count() == 1
    loaded_case = adapter.case(case_obj['_id'])

    assert loaded_case['_id'] == case_obj['_id']

    assert len(loaded_case['panels']) > 0

    for panel in loaded_case['panels']:
        assert panel['display_name']


def test_load_case_rank_model_version(case_obj, panel_database):
    adapter = panel_database

    # GIVEN a database with institute, user, genes, panel but no cases
    assert adapter.gene_panels().count() > 0
    assert adapter.users().count() > 0
    assert adapter.institutes().count() > 0

    # WHEN loading a case
    adapter._add_case(case_obj)

    # THEN assert that the case have been loaded with rank_model
    loaded_case = adapter.case(case_obj['_id'])

    assert loaded_case['rank_model_version'] == case_obj['rank_model_version']


def test_load_case_delivery_report(case_obj, panel_database):
    adapter = panel_database

    # GIVEN a database with institute, user, genes, panel but no cases
    assert adapter.gene_panels().count() > 0
    assert adapter.users().count() > 0
    assert adapter.institutes().count() > 0

    # WHEN loading a case
    adapter._add_case(case_obj)

    # THEN assert that the case have been loaded with delivery_report
    loaded_case = adapter.case(case_obj['_id'])

    assert loaded_case['delivery_report'] == case_obj['delivery_report']



from scout.build.disease import build_disease_term
from scout.models.phenotype_term import DiseaseTerm

def test_build_disease_term(adapter):
    ## GIVEN some disease info and a adapter with a gene
    disease_info = {
        'mim_number': 615349,
        'description': "EHLERS-DANLOS SYNDROME, PROGEROID TYPE, 2",
        'hgnc_symbols': set(['B3GALT6']),
        'inheritance': set(['AR']),
    }
    alias_genes = {}
    alias_genes['B3GALT6'] = {
            'true': 17978,
            'ids': [17978],
        }
    
    ## WHEN building the disease term
    disease_obj = build_disease_term(disease_info, alias_genes)
    
    ## THEN assert the term is on the correct format
    
    assert disease_obj['_id'] == disease_obj['disease_id'] == "OMIM:615349"
    assert disease_obj['inheritance'] == ['AR']
    assert disease_obj['genes'] == [17978]
    assert disease_obj['source'] == 'OMIM'
    
    assert isinstance(disease_obj, DiseaseTerm)


# -*- coding: utf-8 -*-
import logging

from scout.exceptions.config import ConfigError

log = logging.getLogger(__name__)


def check_panels(adapter, panels, default_panels=None):
    """Make sure that the gene panels exist in the database
        Also check if the default panels are defined in gene panels

        Args:
            adapter(MongoAdapter)
            panels(list(str)): A list with panel names

        Returns:
            panels_exists(bool)
    """
    default_panels = default_panels or []
    panels_exist = True
    for panel in default_panels:
        if panel not in panels:
            log.warning("Default panels have to be defined in panels")
            panels_exist = False
    for panel in panels:
        if not adapter.gene_panel(panel):
            log.warning("Panel {} does not exist in database".format(panel))
            panels_exist = False
    return panels_exist


def load_region(adapter, case_id, hgnc_id=None, chrom=None, start=None, end=None):
    """Load all variants in a region defined by a HGNC id

    Args:
        adapter (MongoAdapter)
        case_id (str): Case id
        hgnc_id (int): If all variants from a gene should be uploaded
        chrom (str): If variants from coordinates should be uploaded
        start (int): Start position for region
        end (int): Stop position for region
    """
    if hgnc_id:
        gene_obj = adapter.hgnc_gene(hgnc_id)
        if not gene_obj:
            ValueError("Gene {} does not exist in database".format(hgnc_id))
        chrom = gene_obj['chromosome']
        start = gene_obj['start']
        end = gene_obj['end']

    case_obj = adapter.case(case_id=case_id)
    if not case_obj:
        raise ValueError("Case {} does not exist in database".format(case_id))

    log.info("Load clinical SNV variants for case: {0} region: chr {1}, start"
             " {2}, end {3}".format(case_obj['_id'], chrom, start, end))

    adapter.load_variants(case_obj=case_obj, variant_type='clinical',
                          category='snv', chrom=chrom, start=start, end=end)

    vcf_sv_file = case_obj['vcf_files'].get('vcf_sv')
    if vcf_sv_file:
        log.info("Load clinical SV variants for case: {0} region: chr {1}, "
                 "start {2}, end {3}".format(case_obj['_id'], chrom, start, end))
        adapter.load_variants(case_obj=case_obj, variant_type='clinical',
                              category='sv', chrom=chrom, start=start, end=end)

    vcf_str_file = case_obj['vcf_files'].get('vcf_str')
    if vcf_str_file: 
        log.info("Load clinical STR variants for case: {0} region: chr {1}, "
                 "start {2}, end {3}".format(case_obj['_id'], chrom, start, end))
        adapter.load_variants(case_obj=case_obj, variant_type='clinical',
                              category='str', chrom=chrom, start=start, end=end)

    if case_obj['is_research']:
        log.info("Load research SNV variants for case: {0} region: chr {1}, "
                 "start {2}, end {3}".format(case_obj['_id'], chrom, start, end))
        adapter.load_variants(case_obj=case_obj, variant_type='research',
                              category='snv', chrom=chrom, start=start, end=end)

        vcf_sv_research = case_obj['vcf_files'].get('vcf_sv_research')
        if vcf_sv_research:
            log.info("Load research SV variants for case: {0} region: chr {1},"
                     " start {2}, end {3}".format(case_obj['_id'], chrom, start, end))
            adapter.load_variants(case_obj=case_obj, variant_type='research',
                                  category='sv', chrom=chrom, start=start, end=end)


def load_scout(adapter, config, ped=None, update=False):
    """Load a new case from a Scout config.

        Args:
            adapter(MongoAdapter)
            config(dict): loading info
            ped(Iterable(str)): Pedigree ingformation
            update(bool): If existing case should be updated

    """
    log.info("Check that the panels exists")
    if not check_panels(adapter, config.get('gene_panels', []),
                        config.get('default_gene_panels')):
        raise ConfigError("Some panel(s) does not exist in the database")
    case_obj = adapter.load_case(config, update=update)
    return case_obj



import pytest
from scout.build.genes.exon import build_exon

def test_build_exon():
    ## GIVEN a dictionary with exon information
    exon_info = {
        "exon_id": '1',
        "chrom": '1', 
        "start": 10, 
        "end": 100,     
        "transcript": '12',
        "hgnc_id": 11,
        "rank": 2, # Order of exon in transcript
    }
    
    ## WHEN building a exon object
    exon_obj = build_exon(exon_info)
    
    ## THEN assert that a dictionary is returned
    
    assert isinstance(exon_obj, dict)

def test_build_exon_no_hgnc():
    ## GIVEN a dictionary with exon information
    exon_info = {
        "exon_id": '1',
        "chrom": '1', 
        "start": 10, 
        "end": 100,     
        "transcript": '12',
        "rank": 2, # Order of exon in transcript
    }
    
    ## WHEN building a exon object
    with pytest.raises(KeyError):
        ## THEN assert that a exception is raised since there is no hgnc_id
        exon_obj = build_exon(exon_info)
    


# -*- coding: utf-8 -*-
from scout.parse.variant import parse_variant
from scout.exceptions import VcfError


def test_parse_minimal(one_variant, case_obj):
    """Test to parse a minimal variant"""
    parsed_variant = parse_variant(one_variant, case_obj, variant_type='clinical')
    assert parsed_variant['position'] == int(one_variant.POS)
    assert parsed_variant['category'] == 'snv'


def test_parse_with_header(one_variant, case_obj, rank_results_header):
    """docstring for test_parse_all_variants"""
    parsed_variant = parse_variant(one_variant, case_obj,
                                   rank_results_header=rank_results_header)

    assert parsed_variant['chromosome'] == '1'
    assert parsed_variant['rank_result']['Consequence'] == 1


def test_parse_small_sv(one_sv_variant, case_obj):
    parsed_variant = parse_variant(one_sv_variant, case_obj)

    assert parsed_variant['category'] == 'sv'
    assert parsed_variant['sub_category'] == one_sv_variant.INFO['SVTYPE'].lower()
    assert parsed_variant['position'] == int(one_sv_variant.POS)


def test_parse_many_snvs(variants, case_obj):
    """docstring for test_parse_all_variants"""

    for variant in variants:
        parsed_variant = parse_variant(variant, case_obj)
        assert parsed_variant['chromosome'] == variant.CHROM


def test_parse_many_svs(sv_variants, case_obj):
    """docstring for test_parse_all_variants"""

    for variant in sv_variants:
        try:
            parsed_variant = parse_variant(variant, case_obj)
        except VcfError:
            for info in variant['info_dict']:
                print(info, variant['info'])
            assert False
        assert parsed_variant['chromosome'] == variant.CHROM


def test_parse_cadd(variants, case_obj):
    # GIVEN some parsed variant dicts
    for variant in variants:
        # WHEN score is present
        if 'CADD' in variant.INFO:
            cadd_score = float(variant.INFO['CADD'])
            parsed_variant = parse_variant(variant, case_obj)
            # THEN make sure that the cadd score is parsed correct
            assert parsed_variant['cadd_score'] == cadd_score



# -*- coding: utf-8 -*-
from pprint import pprint as pp
import pytest
from scout.server.blueprints.cases.controllers import get_sanger_unevaluated

def test_get_sanger_unevaluated(real_populated_database, variant_objs, institute_obj, case_obj, user_obj):
    """Test get all sanger ordered but not evaluated for an institute"""

    adapter = real_populated_database

    ## Prepare for variants for Sanger assignment
    # Check that variant collections is empty
    assert adapter.variant_collection.find().count() == 0

    # Adding a number of variants to the empty database
    for index, variant_obj in enumerate(variant_objs):
        adapter.load_variant(variant_obj)

    # Check that variant collections is NOT empty
    assert adapter.variant_collection.find().count() > 0

    # Collect 2 variants from the database
    test_variants = list(adapter.variant_collection.find().limit(2))

    ## Prepare for order Sanger:
    # GIVEN a populated database
    institute = adapter.institute(
        institute_id=institute_obj['internal_id']
    )
    assert institute

    case = adapter.case(
        case_id=case_obj['_id']
    )
    assert case

    user = adapter.user(
        email=user_obj['email']
    )
    assert user

    link = 'orderSangerlink'

    # Order Sanger for both variants
    for variant in test_variants:

        # Assert tha variant has NO Sanger ordered
        assert variant.get('sanger_ordered') == None

        # Assert that variant is not validated
        assert variant.get('validation') == None

        # Order Sanger for variant
        updated_variant = adapter.order_verification(
            institute=institute,
            case=case,
            user=user,
            link=link,
            variant=variant
        )

        # Assert that variant has Sanger ordered now
        assert updated_variant['sanger_ordered']

    # Assert that 4 events were created in events collection (Sanger ordered case X2 + Sanger ordered variant X2)
    assert adapter.event_collection.find().count() == 4

    # Test that the Sanger ordered but not validated for the institute are 2
    # sanger_unevaluated should look like this: [{ 'case_id': [var1, var2] }]
    sanger_unevaluated = get_sanger_unevaluated(adapter, institute['_id'], user_obj['email'])
    assert len(sanger_unevaluated[0][case_obj['display_name']]) == 2

    # Set one of the two variants as validated
    adapter.variant_collection.find_one_and_update(
        {'_id': test_variants[0]['_id']},
        {'$set': {'validation': 'False positive'}},
    )

    # Test that now the Sanger ordered but not validated is only one
    # sanger_unevaluated should look like this: [{ 'case_id': [var2] }]
    sanger_unevaluated = get_sanger_unevaluated(adapter, institute['_id'], user_obj['email'])
    pp(sanger_unevaluated)
    assert len(sanger_unevaluated[0][case_obj['display_name']]) == 1




def build_genotype(gt_call):
    """Build a genotype call
    
    Args:
        gt_call(dict)
    
    Returns:
        gt_obj(dict)
    
    gt_call = dict(
        sample_id = str,
        display_name = str,
        genotype_call = str,
        allele_depths = list, # int
        read_depth = int,
        genotype_quality = int,
    )
    
    
    """
    gt_obj = dict(
        sample_id = gt_call['individual_id'],
        display_name = gt_call['display_name'],
        genotype_call = gt_call['genotype_call'],
        allele_depths = [gt_call['ref_depth'], gt_call['alt_depth']],
        read_depth = gt_call['read_depth'],
        genotype_quality = gt_call['genotype_quality']
    )
    
    return gt_obj


from scout.constants import CONSERVATION

def parse_conservations(variant):
    """Parse the conservation predictors
    
        Args:
            variant(dict): A variant dictionary
        
        Returns:
            conservations(dict): A dictionary with the conservations
    """
    conservations = {}
    
    conservations['gerp'] = parse_conservation(
                                            variant, 
                                            'GERP++_RS_prediction_term'
                                        )
    conservations['phast'] = parse_conservation(
                                            variant,
                                            'phastCons100way_vertebrate_prediction_term'
                                        )
    conservations['phylop'] = parse_conservation(
                                            variant,
                                            'phyloP100way_vertebrate_prediction_term'
                                        )
    return conservations

def parse_conservation(variant, info_key):
    """Get the conservation prediction
    
        Args:
            variant(dict): A variant dictionary
            info_key(str)
        
        Returns:
            conservations(list): List of censervation terms
    """
    raw_annotation = variant.INFO.get(info_key)
    conservations = []
    
    if raw_annotation:
        for term in raw_annotation.split(','):
            term = term.strip()
            if term in CONSERVATION:
                conservations.append(term)
    
    return conservations



from pprint import pprint as pp
from scout.load.hgnc_gene import load_hgnc_genes

def test_load_hgnc_genes(adapter, genes37_handle, hgnc_handle, exac_handle,
          mim2gene_handle, genemap_handle, hpo_genes_handle):
    # GIVEN a empty database
    assert adapter.all_genes().count() == 0
    
    # WHEN inserting a number of genes
    gene_objects = load_hgnc_genes(adapter, 
        ensembl_lines=genes37_handle, 
        hgnc_lines=hgnc_handle, 
        exac_lines=exac_handle, 
        mim2gene_lines=mim2gene_handle,
        genemap_lines=genemap_handle, 
        hpo_lines=hpo_genes_handle, 
        build='37'
    )
    
    nr_genes = 0
    for gene_info in gene_objects:
        if gene_info.get('chromosome'):
            nr_genes += 1
    
    # THEN assert all genes have been added to the database
    assert adapter.all_genes().count() == nr_genes
    
    # THEN assert that the last gene was loaded
    assert adapter.hgnc_gene(gene_info['hgnc_id'])



from scout.constants import (CYTOBANDS, BND_ALT_PATTERN, CHR_PATTERN)

def get_cytoband_coordinates(chrom, pos):
    """Get the cytoband coordinate for a position

    Args:
        chrom(str)
        pos(int)

    Returns:
        coordinate(str)
    """
    coordinate = ""

    if chrom in CYTOBANDS:
        for interval in CYTOBANDS[chrom][pos]:
            coordinate = interval.data

    return coordinate

def get_sub_category(alt_len, ref_len, category, svtype=None):
    """Get the subcategory for a VCF variant

    The sub categories are:
        'snv', 'indel', 'del', 'ins', 'dup', 'bnd', 'inv'

    Args:
        alt_len(int)
        ref_len(int)
        category(str)
        svtype(str)

    Returns:
        subcategory(str)
    """
    subcategory = ''

    if category in ('snv', 'indel', 'cancer'):
        if ref_len == alt_len:
            subcategory = 'snv'
        else:
            subcategory = 'indel'
    elif category == 'sv':
        subcategory = svtype

    return subcategory

def get_length(alt_len, ref_len, category, pos, end, svtype=None, svlen=None):
    """Return the length of a variant

    Args:
        alt_len(int)
        ref_len(int)
        category(str)
        svtype(str)
        svlen(int)
    """
    # -1 would indicate uncertain length
    length = -1
    if category in ('snv', 'indel', 'cancer'):
        if ref_len == alt_len:
            length = alt_len
        else:
            length = abs(ref_len - alt_len)

    elif category == 'sv':
        if svtype == 'bnd':
            length = int(10e10)
        else:
            if svlen:
                length = abs(int(svlen))
            # Some software does not give a length but they give END
            elif end:
                if end != pos:
                    length = end - pos
    return length

def get_end(pos, alt, category, snvend=None, svend=None, svlen=None):
    """Return the end coordinate for a variant

    Args:
        pos(int)
        alt(str)
        category(str)
        snvend(str)
        svend(int)
        svlen(int)

    Returns:
        end(int)
    """
    # If nothing is known we set end to be same as start
    end = pos
    # If variant is snv or indel we know that cyvcf2 can handle end pos
    if category in ('snv', 'indel', 'cancer'):
        end = snvend

    # With SVs we have to be a bit more careful
    elif category == 'sv':
        # The END field from INFO usually works fine
        end = svend

        # For some cases like insertions the callers set end to same as pos
        # In those cases we can hope that there is a svlen...
        if svend == pos:
            if svlen:
                end = pos + svlen
        # If variant is 'BND' they have ':' in alt field
        # Information about other end is in the alt field
        if ':' in alt:
            match = BND_ALT_PATTERN.match(alt)
            if match:
                end = int(match.group(2))

    return end

def parse_coordinates(variant, category):
    """Find out the coordinates for a variant

    Args:
        variant(cyvcf2.Variant)

    Returns:
        coordinates(dict): A dictionary on the form:
        {
            'position':<int>,
            'end':<int>,
            'end_chrom':<str>,
            'length':<int>,
            'sub_category':<str>,
            'mate_id':<str>,
            'cytoband_start':<str>,
            'cytoband_end':<str>,
        }
    """
    ref = variant.REF

    if variant.ALT:
        alt = variant.ALT[0]
    if category=="str" and not variant.ALT:
        alt = '.'

    chrom_match = CHR_PATTERN.match(variant.CHROM)
    chrom = chrom_match.group(2)

    svtype = variant.INFO.get('SVTYPE')
    if svtype:
        svtype = svtype.lower()

    mate_id = variant.INFO.get('MATEID')

    svlen = variant.INFO.get('SVLEN')

    svend = variant.INFO.get('END')
    snvend = int(variant.end)

    position = int(variant.POS)

    ref_len = len(ref)
    alt_len = len(alt)

    sub_category = get_sub_category(alt_len, ref_len, category, svtype)
    end = get_end(position, alt, category, snvend, svend)

    length = get_length(alt_len, ref_len, category, position, end, svtype, svlen)
    end_chrom = chrom

    if sub_category == 'bnd':
        if ':' in alt:
            match = BND_ALT_PATTERN.match(alt)
            # BND will often be translocations between different chromosomes
            if match:
                other_chrom = match.group(1)
                match = CHR_PATTERN.match(other_chrom)
                end_chrom = match.group(2)

    cytoband_start = get_cytoband_coordinates(chrom, position)
    cytoband_end = get_cytoband_coordinates(end_chrom, end)

    coordinates = {
        'position': position,
        'end': end,
        'length': length,
        'sub_category': sub_category,
        'mate_id': mate_id,
        'cytoband_start': cytoband_start,
        'cytoband_end': cytoband_end,
        'end_chrom': end_chrom,
    }


    return coordinates



#!/usr/bin/env python
# encoding: utf-8
import logging

import click

from scout.load.all import load_region

LOG = logging.getLogger(__name__)


@click.command('region', short_help='Load variants from region')
@click.option('--hgnc-id', type=int,
              help="Use a existing hgnc id to define the region", )
@click.option('--case-id', required=True, help="Id of case")
@click.option('-c', '--chromosome')
@click.option('-s', '--start', type=int)
@click.option('-e', '--end', type=int)
@click.pass_context
def region(context, hgnc_id, case_id, chromosome, start, end):
    """Load all variants in a region to a existing case"""
    adapter = context.obj['adapter']
    load_region(
        adapter=adapter, case_id=case_id, hgnc_id=hgnc_id, chrom=chromosome, start=start, end=end
    )



from scout.parse.omim import (parse_omim_line, parse_genemap2, parse_mim_titles, 
                              parse_mim2gene, get_mim_phenotypes)

GENEMAP_LINES = [
    "# Copyright (c) 1966-2016 Johns Hopkins University. Use of this"\
    " file adheres to the terms specified at https://omim.org/help/agreement.\n",
    "# Generated: 2017-02-02\n",
    "# See end of file for additional documentation on specific fields\n",
    "# Chromosome\tGenomic Position Start\tGenomic Position End\tCyto"\
    " Location\tComputed Cyto Location\tMim Number\tGene Symbols\tGene Name"\
    "\tApproved Symbol\tEntrez Gene ID\tEnsembl Gene ID\tComments\t"\
    "Phenotypes\tMouse Gene Symbol/ID\n",
    "chr1\t1232248\t1235040\t1p36.33\t\t615291\tB3GALT6, SEMDJL1, EDSP2\t"\
    "UDP-Gal:beta-Gal beta-1,3-galactosyltransferase polypeptide 6\tB3GALT"\
    "6\t126792\tENSG00000176022\t\tEhlers-Danlos syndrome, progeroid type,"\
    " 2, 615349 (3), Autosomal recessive; Spondyloepimetaphyseal dysplasia"\
    " with joint laxity, type 1, with or without fractures, 271640 (3),"\
    " Autosomal recessive\tB3galt6 (MGI:2152819)\n",
]

MIM2GENE_LINES = [
    "# Copyright (c) 1966-2016 Johns Hopkins University. Use of this file "\
    "adheres to the terms specified at https://omim.org/help/agreement.\n",
    "# Generated: 2017-02-02\n",
    "# This file provides links between the genes in OMIM and other gene"\
    " identifiers.\n",
    "# THIS IS NOT A TABLE OF GENE-PHENOTYPE RELATIONSHIPS.\n"
    "# MIM Number\tMIM Entry Type (see FAQ 1.3 at https://omim.org/help/faq)\t"\
    "Entrez Gene ID (NCBI)\tApproved Gene Symbol (HGNC)\tEnsembl Gene ID (Ensembl)\n",
    "615291\tgene\t126792\tB3GALT6\tENSG00000176022,ENST00000379198",
    "615349\tphenotype",
    "271640\tphenotype",
]

def test_parse_omim_line():
    ## GIVEN a header and a line
    header = ['a', 'b', 'c']
    line = '1\t2\t3'
    ## WHEN parsing the omim line
    res = parse_omim_line(line, header)
    
    ## THEN assert a dict was built by the header and the line
    assert res['a'] == '1'
    assert res['b'] == '2'
    assert res['c'] == '3'

def test_parse_genemap():
    
    for res in parse_genemap2(GENEMAP_LINES):
        assert res['Chromosome'] == 'chr1'
        assert res['mim_number'] == 615291
        assert res['hgnc_symbol'] == 'B3GALT6'
        assert res['inheritance'] == set(['AR'])
        for phenotype in res['phenotypes']:
            assert phenotype['mim_number']
            assert phenotype['inheritance']

def test_parse_genemap_file(genemap_handle):
    for i,res in enumerate(parse_genemap2(genemap_handle)):
        assert 'mim_number' in res
    
    assert i > 0

def test_parse_mim2gene():
    ## GIVEN some lines from a mim2gene file
    mim2gene_info = parse_mim2gene(MIM2GENE_LINES)
    
    ## WHEN parsing the lines
    first_entry = next(mim2gene_info)
    
    ## ASSERT that they are correctly parsed
    
    # First entry is a gene so it should have a hgnc symbol
    assert first_entry['mim_number'] == 615291
    assert first_entry['entry_type'] == 'gene'
    assert first_entry['hgnc_symbol'] == 'B3GALT6'

def test_parse_mim2gene_file(mim2gene_handle):
    # Just check that the file exists and that some result is given
    for i,res in enumerate(parse_mim2gene(mim2gene_handle)):
        assert 'mim_number' in res
    assert i > 0

def test_get_mim_phenotypes():
    ## GIVEN a small testdata set
    
    # This will return a dictionary with mim number as keys and
    # phenotypes as values
    
    ## WHEN parsing the phenotypes
    phenotypes = get_mim_phenotypes(genemap_lines=GENEMAP_LINES)

    ## THEN assert they where parsed in a correct way
    
    # There was only one line in GENEMAP_LINES that have two phenotypes
    # so we expect that there should be two phenotypes

    assert len(phenotypes) == 2

    term = phenotypes[615349]

    assert term['inheritance'] == set(['AR'])
    assert term['hgnc_symbols'] == set(['B3GALT6'])

def test_get_mim_phenotypes_file(genemap_handle):
    phenotypes = get_mim_phenotypes(genemap_lines=genemap_handle)
    
    for i, mim_nr in enumerate(phenotypes):
        assert phenotypes[mim_nr]['mim_number']

    assert i > 0
        


from pprint import pprint as pp

def add_gene_links(gene_obj, build=37):
    """Update a gene object with links

    Args:
        gene_obj(dict)
        build(int)

    Returns:
        gene_obj(dict): gene_obj updated with many links
    """
    try:
        build = int(build)
    except ValueError:
        build = 37
    # Add links that use the hgnc_id
    hgnc_id = gene_obj['hgnc_id']

    gene_obj['hgnc_link'] = genenames(hgnc_id)
    gene_obj['omim_link'] = omim(hgnc_id)
    # Add links that use ensembl_id
    if not 'ensembl_id' in gene_obj:
        ensembl_id = gene_obj.get('common',{}).get('ensembl_id')
    else:
        ensembl_id = gene_obj['ensembl_id']
    ensembl_37_link = ensembl(ensembl_id, build=37)
    ensembl_38_link = ensembl(ensembl_id, build=38)
    gene_obj['ensembl_37_link'] = ensembl_37_link
    gene_obj['ensembl_38_link'] = ensembl_38_link
    gene_obj['ensembl_link'] = ensembl_37_link
    if build == 38:
        gene_obj['ensembl_link'] = ensembl_38_link
    gene_obj['hpa_link'] = hpa(ensembl_id)
    gene_obj['string_link'] = string(ensembl_id)
    gene_obj['reactome_link'] = reactome(ensembl_id)
    gene_obj['expression_atlas_link'] = expression_atlas(ensembl_id)
    gene_obj['exac_link'] = exac(ensembl_id)
    # Add links that use entrez_id
    gene_obj['entrez_link'] = entrez(gene_obj.get('entrez_id'))
    # Add links that use omim id
    gene_obj['omim_link'] = omim(gene_obj.get('omim_id'))
    # Add links that use hgnc_symbol
    gene_obj['ppaint_link'] = ppaint(gene_obj['hgnc_symbol'])
    # Add links that use vega_id
    gene_obj['vega_link'] = vega(gene_obj.get('vega_id'))
    # Add links that use ucsc link
    gene_obj['ucsc_link'] = ucsc(gene_obj.get('ucsc_id'))

def genenames(hgnc_id):
    link = "https://www.genenames.org/cgi-bin/gene_symbol_report?hgnc_id=HGNC:{}"
    if not hgnc_id:
        return None
    return link.format(hgnc_id)

def omim(omim_id):
    link = "https://www.omim.org/entry/{}"
    if not omim_id:
        return None
    return link.format(omim_id)

def ensembl(ensembl_id, build=37):

    link = "http://grch37.ensembl.org/Homo_sapiens/Gene/Summary?db=core;g={}"
    if build == 38:
        link = "http://ensembl.org/Homo_sapiens/Gene/Summary?db=core;g={}"
    if not ensembl_id:
        return None

    return link.format(ensembl_id)

def exac(ensembl_id):

    link = "http://exac.broadinstitute.org/gene/{}"
    if not ensembl_id:
        return None

    return link.format(ensembl_id)

def hpa(ensembl_id):
    link = "http://www.proteinatlas.org/search/{}"

    if not ensembl_id:
        return None

    return link.format(ensembl_id)

def string(ensembl_id):
    link = ("http://string-db.org/newstring_cgi/show_network_"
            "section.pl?identifier={}")

    if not ensembl_id:
        return None

    return link.format(ensembl_id)

def reactome(ensembl_id):
    link = ("http://www.reactome.org/content/query?q={}&species=Homo+sapiens"
            "&species=Entries+without+species&cluster=true")
    if not ensembl_id:
        return None

    return link.format(ensembl_id)

def expression_atlas(ensembl_id):
    link = "https://www.ebi.ac.uk/gxa/genes/{}"
    if not ensembl_id:
        return None

    return link.format(ensembl_id)

def entrez(entrez_id):
    link = "https://www.ncbi.nlm.nih.gov/gene/{}"

    if not entrez_id:
        return None

    return link.format(entrez_id)

def ppaint(hgnc_symbol):
    link = "https://pecan.stjude.cloud/proteinpaint/{}"

    if not hgnc_symbol:
        return None

    return link.format(hgnc_symbol)

def vega(vega_id):
    link = "http://vega.archive.ensembl.org/Homo_sapiens/Gene/Summary?db=core;g=OTTHUMG00000018506{}"

    if not vega_id:
        return None

    return link.format(vega_id)

def ucsc(ucsc_id, build=37):
    link = "https://genome.ucsc.edu/cgi-bin/hgGene?db=hg{0}&hgg_chrom=chr10&hgg_gene={0}"

    if not ucsc_id:
        return None

    return link.format(build,ucsc_id)

def add_tx_links(tx_obj, build=37):
    try:
        build = int(build)
    except ValueError:
        build = 37

    ensembl_id = tx_obj.get('ensembl_transcript_id', tx_obj.get('transcript_id'))
    ensembl_37_link = ensembl_tx(ensembl_id, 37)
    ensembl_38_link = ensembl_tx(ensembl_id, 38)

    tx_obj['ensembl_37_link'] = ensembl_37_link
    tx_obj['ensembl_38_link'] = ensembl_38_link
    tx_obj['ensembl_link'] = ensembl_37_link
    if build == 38:
        tx_obj['ensembl_link'] = ensembl_38_link

    refseq_links = []
    if tx_obj.get('refseq_id'):
        refseq_id = tx_obj['refseq_id']
        refseq_links.append(
            {
                'link': refseq(refseq_id),
                'id': refseq_id
            }
        )
    tx_obj['refseq_links'] = refseq_links
    tx_obj['swiss_prot_link'] = swiss_prot(tx_obj.get('swiss_prot'))
    tx_obj['pfam_domain_link'] = pfam(tx_obj.get('pfam_domain'))
    tx_obj['prosite_profile_link'] = prosite(tx_obj.get('prosite_profile'))
    tx_obj['smart_domain_link'] = smart(tx_obj.get('smart_domain'))

    return tx_obj


##  Transcript links
def refseq(refseq_id):
    link = "http://www.ncbi.nlm.nih.gov/nuccore/{}"
    if not refseq_id:
        return None

    return link.format(refseq_id)

def ensembl_tx(ens_tx_id, build=37):
    link = ("http://grch37.ensembl.org/Homo_sapiens/"
                    "Gene/Summary?t={}")

    if build == 38:
        link = ("http://ensembl.org/Homo_sapiens/"
                        "Gene/Summary?t={}")
    if not ens_tx_id:
        return None

    return link.format(ens_tx_id)

def swiss_prot(swiss_prot_id):
    link = "http://www.uniprot.org/uniprot/{}"

    if not swiss_prot_id:
        return None

    return link.format(swiss_prot_id)

def pfam(pfam_domain):
    link = "http://pfam.xfam.org/family/{}"

    if not pfam_domain:
        return None

    return link.format(pfam_domain)

def prosite(prosat_profile):
    link = ("http://prosite.expasy.org/cgi-bin/prosite/"
                                      "prosite-search-ac?{}")
    if not prosat_profile:
        return None

    return link.format(prosat_profile)

def smart(smart_domain):
    link = "http://smart.embl.de/smart/search.cgi?keywords={}"

    if not smart_domain:
        return None

    return link.format(smart_domain)



import logging
import click

LOG = logging.getLogger(__name__)


@click.command('aliases', short_help='Display genes by aliases')
@click.option('-b', '--build', default='37', type=click.Choice(['37', '38']))
@click.option('-s', '--symbol', help='hgnc symbol')
@click.pass_context
def aliases(context, build, symbol):
    """Show all alias symbols and how they map to ids"""
    LOG.info("Running scout view aliases")
    adapter = context.obj['adapter']
    
    if symbol:
        alias_genes = {}
        res = adapter.gene_by_alias(symbol, build=build)
        for gene_obj in res:
            hgnc_id = gene_obj['hgnc_id']
            # Collect the true symbol given by hgnc
            hgnc_symbol = gene_obj['hgnc_symbol']
            # Loop aver all aliases
            for alias in gene_obj['aliases']:
                true_id = None
                # If the alias is the same as hgnc symbol we know the true id
                if alias == hgnc_symbol:
                    true_id = hgnc_id
                # If the alias is already in the list we add the id
                if alias in alias_genes:
                    alias_genes[alias]['ids'].add(hgnc_id)
                    if true_id:
                        alias_genes[alias]['true'] = hgnc_id
                else:
                    alias_genes[alias] = {
                        'true': hgnc_id,
                        'ids': set([hgnc_id])
                    }

    else:
        alias_genes = adapter.genes_by_alias(build=build)
    
    if len(alias_genes) == 0:
        LOG.info("No gene found for build %s", build)
        return

    click.echo("#hgnc_symbol\ttrue_id\thgnc_ids")
    for alias_symbol in alias_genes:
        info = alias_genes[alias_symbol]
        # pp(info)
        click.echo("{0}\t{1}\t{2}\t".format(
            alias_symbol,
            (alias_genes[alias_symbol]['true'] or 'None'),
            ', '.join([str(gene_id) for gene_id in alias_genes[alias_symbol]['ids']])
        )
        )




import logging

from scout.constants import (REV_PHENOTYPE_MAP, REV_SEX_MAP, ANALYSIS_TYPES)
from scout.exceptions import PedigreeError

log = logging.getLogger(__name__)


def build_individual(ind):
    """Build a Individual object

        Args:
            ind (dict): A dictionary with individual information

        Returns:
            ind_obj (dict): A Individual object

        dict(
            individual_id = str, # required
            display_name = str,
            sex = str,
            phenotype = int,
            father = str, # Individual id of father
            mother = str, # Individual id of mother
            capture_kits = list, # List of names of capture kits
            bam_file = str, # Path to bam file
            vcf2cytosure = str, # Path to CGH file
            analysis_type = str, # choices=ANALYSIS_TYPES
        )

    """

    try:
        ind_obj = dict(
            individual_id=ind['individual_id']
        )
        log.info("Building Individual with id:{0}".format(ind['individual_id']))
    except KeyError as err:
        raise PedigreeError("Individual is missing individual_id")

    ind_obj['display_name'] = ind.get('display_name', ind_obj['individual_id'])

    sex = ind.get('sex', 'unknown')
    # Convert sex to .ped
    try:
        # Check if sex is coded as an integer
        int(sex)
        ind_obj['sex'] = str(sex)
    except ValueError as err:
        try:
            # Sex are numbers in the database
            ind_obj['sex'] = REV_SEX_MAP[sex]
        except KeyError as err:
            raise(PedigreeError("Unknown sex: %s" % sex))

    phenotype = ind.get('phenotype', 'unknown')
    # Make the phenotype integers
    try:
        ped_phenotype = REV_PHENOTYPE_MAP[phenotype]
        if ped_phenotype == -9:
            ped_phenotype = 0
        ind_obj['phenotype'] = ped_phenotype
    except KeyError as err:
        raise(PedigreeError("Unknown phenotype: %s" % phenotype))

    ind_obj['father'] = ind.get('father')
    ind_obj['mother'] = ind.get('mother')
    ind_obj['capture_kits'] = ind.get('capture_kits', [])
    ind_obj['bam_file'] = ind.get('bam_file')
    ind_obj['mt_bam'] = ind.get('mt_bam')
    ind_obj['vcf2cytosure'] = ind.get('vcf2cytosure')
    ind_obj['confirmed_sex'] = ind.get('confirmed_sex')
    ind_obj['confirmed_parent'] = ind.get('confirmed_parent')
    ind_obj['predicted_ancestry'] = ind.get('predicted_ancestry')

    # Check if the analysis type is ok
    # Can be anyone of ('wgs', 'wes', 'mixed', 'unknown')
    analysis_type = ind.get('analysis_type', 'unknown')
    if not analysis_type in ANALYSIS_TYPES:
        raise PedigreeError("Analysis type %s not allowed", analysis_type)
    ind_obj['analysis_type'] = analysis_type

    return ind_obj



from scout.parse.variant.rank_score import parse_rank_score

def test_parse_rank_score():
    rank_scores_info = "123:10"
    variant_score = 10.0
    family_id = '123'
    
    parsed_rank_score = parse_rank_score(rank_scores_info, family_id)
    
    assert variant_score == parsed_rank_score

def test_parse_rank_score_no_score():
    rank_scores_info = ""
    family_id = '123'
    
    parsed_rank_score = parse_rank_score(rank_scores_info, family_id)
    
    assert parsed_rank_score == None


# def test_parse_rank_scores(variants, parsed_case):
#     """docstring for test_parse_rank_score"""
#     case_id = parsed_case['display_name']
#     for variant in variants:
#         rank_scores_dict = variant['rank_scores']
#
#         rank_score = rank_scores_dict[case_id]
#
#         assert float(rank_score) == parse_rank_score(variant, case_id)


# -*- coding: utf-8 -*-
import logging

from scout.constants import SO_TERMS
from pprint import pprint as pp

LOG = logging.getLogger(__name__)

def parse_transcripts(raw_transcripts, allele=None):
    """Parse transcript information from VCF variants

    Args:
        raw_transcripts(iterable(dict)): An iterable with raw transcript 
                                         information

    Yields:
        transcript(dict) A dictionary with transcript information
    """
    for entry in raw_transcripts:
        transcript = {}
        # There can be several functional annotations for one variant
        functional_annotations = entry.get('CONSEQUENCE', '').split('&')
        transcript['functional_annotations'] = functional_annotations
        # Get the transcript id (ensembl gene id)
        transcript_id = entry.get('FEATURE', '').split(':')[0]
        transcript['transcript_id'] = transcript_id

        # Add the hgnc gene identifiers
        # The HGNC ID is prefered and will be used if it exists
        hgnc_id = entry.get('HGNC_ID')
        if hgnc_id:
            hgnc_id = hgnc_id.split(':')[-1]
            transcript['hgnc_id'] = int(hgnc_id)
        else:
            transcript['hgnc_id'] = None

        hgnc_symbol = entry.get('SYMBOL')
        if hgnc_symbol:
            transcript['hgnc_symbol'] = hgnc_symbol
        else:
            transcript['hgnc_symbol'] = None

        ########### Fill it with the available information ###########

        ### Protein specific annotations ###

        ## Protein ID ##
        transcript['protein_id'] = entry.get('ENSP')
        
        ## Polyphen prediction ##
        polyphen_prediction = entry.get('POLYPHEN')
        # Default is 'unknown'
        prediction_term = 'unknown'
        if polyphen_prediction:
            prediction_term = polyphen_prediction.split('(')[0]
        transcript['polyphen_prediction'] = prediction_term

        ## Sift prediction ##
        # Check with other key if it does not exist
        sift_prediction = entry.get('SIFT')
        # Default is 'unknown'
        prediction_term = 'unknown'
        
        if not sift_prediction:
            sift_prediction = entry.get('SIFT_PRED')
            
        if sift_prediction:
            prediction_term = sift_prediction.split('(')[0]

        transcript['sift_prediction'] = prediction_term
        
        transcript['swiss_prot'] = entry.get('SWISSPROT') or 'unknown'

        if entry.get('DOMAINS', None):
            pfam_domains = entry['DOMAINS'].split('&')

            for annotation in pfam_domains:
                annotation = annotation.split(':')
                domain_name = annotation[0]
                domain_id = annotation[1]
                if domain_name == 'Pfam_domain':
                    transcript['pfam_domain'] = domain_id
                elif domain_name == 'PROSITE_profiles':
                    transcript['prosite_profile'] = domain_id
                elif domain_name == 'SMART_domains':
                    transcript['smart_domain'] = domain_id

        coding_sequence_entry = entry.get('HGVSC', '').split(':')
        protein_sequence_entry = entry.get('HGVSP', '').split(':')

        coding_sequence_name = None
        if len(coding_sequence_entry) > 1:
            coding_sequence_name = coding_sequence_entry[-1]

        transcript['coding_sequence_name'] = coding_sequence_name

        protein_sequence_name = None
        if len(protein_sequence_entry) > 1:
            protein_sequence_name = protein_sequence_entry[-1]

        transcript['protein_sequence_name'] = protein_sequence_name

        transcript['biotype'] = entry.get('BIOTYPE')

        transcript['exon'] = entry.get('EXON')
        transcript['intron'] = entry.get('INTRON')

        if entry.get('STRAND'):
            if entry['STRAND'] == '1':
                transcript['strand'] = '+'
            elif entry['STRAND'] == '-1':
                transcript['strand'] = '-'
        else:
            transcript['strand'] = None

        functional = []
        regional = []
        for annotation in functional_annotations:
            functional.append(annotation)
            regional.append(SO_TERMS[annotation]['region'])

        transcript['functional_annotations'] = functional
        transcript['region_annotations'] = regional

        # Check if the transcript is marked cannonical by vep
        transcript['is_canonical'] = (entry.get('CANONICAL') == 'YES')
        
        # Check if the CADD score is available on transcript level
        cadd_phred = entry.get('CADD_PHRED')
        if cadd_phred:
            transcript['cadd'] = float(cadd_phred)

        # Check frequencies
        # There are different keys for different versions of VEP
        # We only support version 90+
        thousandg_freqs = []
        gnomad_freqs = []
        try:
            # The keys for VEP v90+:
            # 'AF' or '1000GAF' - 1000G all populations combined
            # 'xxx_AF' - 1000G (or NHLBI-ESP) individual populations
            # 'gnomAD_AF' - gnomAD exomes, all populations combined
            # 'gnomAD_xxx_AF' - gnomAD exomes, individual populations
            # 'MAX_AF' - Max of all populations (1000G, gnomAD exomes, ESP)
            # https://www.ensembl.org/info/docs/tools/vep/vep_formats.html
            
            # Loop over all keys to find frequency entries
            for key in entry:
                #All frequencies endswith AF
                if not key.endswith('AF'):
                    continue
                
                value = entry[key]
                if not value:
                    continue
                
                # This is the 1000G max af information
                if (key == 'AF' or key == '1000GAF'):
                    transcript['thousand_g_maf'] = float(value)
                    continue
                
                if key == 'GNOMAD_AF':
                    transcript['gnomad_maf'] = float(value)
                    continue

                if key == 'EXAC_MAX_AF':
                    transcript['exac_max'] = float(value)
                    transcript['exac_maf'] = float(value)
                    continue
                
                if 'GNOMAD' in key:
                    gnomad_freqs.append(float(value))
                
                else:
                    thousandg_freqs.append(float(value))
                
            if thousandg_freqs:
                transcript['thousandg_max'] = max(thousandg_freqs)

            if gnomad_freqs:
                transcript['gnomad_max'] = max(gnomad_freqs)
            
        except Exception as err:
            LOG.debug("Something went wrong when parsing frequencies")
            LOG.debug("Only splitted and normalised VEP v90+ is supported")

        clinsig = entry.get('CLIN_SIG')
        if clinsig:
            transcript['clinsig'] = clinsig.split('&')

        transcript['dbsnp'] = []
        transcript['cosmic'] = []
        variant_ids = entry.get('EXISTING_VARIATION')
        if variant_ids:
            for variant_id in variant_ids.split('&'):
                if variant_id.startswith('rs'):
                    transcript['dbsnp'].append(variant_id)
                elif variant_id.startswith('COSM'):
                    transcript['cosmic'].append(int(variant_id[4:]))

        yield transcript




import pytest

from scout.exceptions import IntegrityError
from scout.build.panel import (build_panel, build_gene)
from datetime import datetime

def test_build_panel_gene(adapter, test_gene):
    adapter.load_hgnc_gene(test_gene)
    ## GIVEN some gene info
    gene_info = {
        'hgnc_id': test_gene['hgnc_id'],
        'inheritance_models': ['AR','AD']
    }
    
    ## WHEN building a gene obj
    gene_obj = build_gene(gene_info, adapter)
    
    ## THEN assert that the object is correct
    
    assert gene_obj['hgnc_id'] == 1
    assert gene_obj['symbol'] == test_gene['hgnc_symbol']
    assert gene_obj['ar'] == True
    assert gene_obj['ad'] == True
    assert 'mt' not in gene_obj 
    

def test_build_panel(institute_database, test_gene):
    ## GIVEN a adapter with a gene and a institute
    adapter = institute_database
    adapter.load_hgnc_gene(test_gene)
    
    panel_info = {
        'panel_id': 'panel1',
        'institute': 'cust000',
        'date': datetime.now(),
        'display_name': 'first panel',
        'genes': [{'hgnc_id':1}],
        'version': 1.0,
    }
    ## WHEN building a gene panel
    panel_obj = build_panel(panel_info, adapter)

    ## THEN assert that the panel was given the right attributes
    assert panel_obj['institute'] == panel_info['institute']
    assert len(panel_info['genes']) == len(panel_obj['genes'])

def test_build_panel_no_id(institute_database, test_gene):
    ## GIVEN a adapter with a gene and a institute
    adapter = institute_database
    adapter.load_hgnc_gene(test_gene)
    
    ## WHEN building a gene panel without panel_name
    panel_info = {
        'institute': 'cust000',
        'date': datetime.now(),
        'display_name': 'first panel',
        'genes': [{'hgnc_id':1}],
        'version': 1.0,
    }
    ## THEN assert a KeyError was raised
    with pytest.raises(KeyError):
        panel_obj = build_panel(panel_info, adapter)

def test_build_panel_no_institute(institute_database, test_gene):
    ## GIVEN a adapter with a gene and a institute
    adapter = institute_database
    adapter.load_hgnc_gene(test_gene)
    
    ## WHEN building a gene panel without institute
    panel_info = {
        'panel_name': 'panel1',
        'date': datetime.now(),
        'display_name': 'first panel',
        'genes': [{'hgnc_id':1}],
        'version': 1.0,
    }
    ## THEN assert a KeyError was raised
    with pytest.raises(KeyError):
        panel_obj = build_panel(panel_info, adapter)

def test_build_panel_no_date(institute_database, test_gene):
    ## GIVEN a adapter with a gene and a institute
    adapter = institute_database
    adapter.load_hgnc_gene(test_gene)
    
    ## WHEN building a gene panel without date
    panel_info = {
        'panel_name': 'panel1',
        'institute': 'cust000',
        'display_name': 'first panel',
        'genes': [{'hgnc_id':1}],
        'version': 1.0,
    }
    ## THEN assert a KeyError was raised
    with pytest.raises(KeyError):
        panel_obj = build_panel(panel_info, adapter)


def test_build_panel_non_existing_insitute(institute_database, test_gene):
    ## GIVEN a adapter with a gene and a institute
    adapter = institute_database
    adapter.load_hgnc_gene(test_gene)
    assert adapter.institute('cust001') is None

    ## WHEN building a gene panel with wrong institute
    panel_info = {
        'panel_name': 'panel1',
        'institute': 'cust0001',
        'date': datetime.now(),
        'display_name': 'first panel',
        'genes': [{'hgnc_id':1}],
        'version': 1.0,
    }
    ## THEN assert that an IntegrityError is raised
    with pytest.raises(IntegrityError):
        panel_obj = build_panel(panel_info, adapter)

